@@ -80,7 +80,7 @@ class MultiTextEntryDialog(gtk.Dialog):
         """
             Returns a list of the values from the added fields
         """
-        return [a.get_text() for a in self.fields]
+        return [unicode(a.get_text(), 'utf-8') for a in self.fields]
 
     def run(self):
         """
@@ -126,7 +126,7 @@ class TextEntryDialog(gtk.Dialog):
         """
             Returns the text value
         """
-        return self.entry.get_text()
+        return unicode(self.entry.get_text(), 'utf-8')
 
     def set_value(self, value):
         """
@@ -140,6 +140,15 @@ class TextEntryDialog(gtk.Dialog):
         self.hide()
         return response
 
+def to_unicode(x, default_encoding=None):
+    if isinstance(x, unicode):
+        return x
+    elif default_encoding and isinstance(x, str):
+        # This unicode constructor only accepts "string or buffer".
+        return unicode(x, default_encoding)
+    else:
+        return unicode(x)
+
 def tup(string, num):
     """
         returns a tuple with the first char of the string repeated 
