@@ -16,7 +16,8 @@
 
 import os, re
 from ConfigParser import SafeConfigParser
-import ConfigParser
+import ConfigParser, gobject
+from xl import common, xlmisc
 
 # ok, I know this isn't secure, but it's better than having passwords in
 # plaintext in the configuration file.
@@ -37,9 +38,11 @@ class XlConfigParser(SafeConfigParser):
         """
         SafeConfigParser.__init__(self)
         self.loc = loc
+        self.writing = False
+        self.dirty = False
 
         sections = ['ui', 'osd', 'lastfm', 'equalizer',\
-            'editor', 'import', 'replaygain']
+            'editor', 'import', 'replaygain', 'proxy', 'playlist']
 
         if os.path.exists(self.loc):
             try:
@@ -65,10 +68,11 @@ class XlConfigParser(SafeConfigParser):
 
     def get_section_key(self, key, plugin=None):
         plugin_str = plugin
+
         if plugin:
             if plugin.endswith(".py"):
                 plugin_str = plugin[:-3]
-        
+
         if plugin_str:
             if plugin_str not in self.sections():
                 self.add_section(plugin=plugin_str)
@@ -91,6 +95,7 @@ class XlConfigParser(SafeConfigParser):
             does not exist in the current settings, the default is returned
         """
         value = None
+
         try:
             value = SafeConfigParser.get(self, *self.get_section_key(key, plugin))
             return value
@@ -166,12 +171,13 @@ class XlConfigParser(SafeConfigParser):
         return value
 
 
-    def get_list(self, key, default=[], plugin=None):
+    def get_list(self, key, default=None, plugin=None):
         """
             Gets a list from the settings.  If default is passed, and the value
             does not exist in the current settings, the default is returned
         """
-        value = self.get(key, plugin)
+
+        value = self.get(key, plugin=plugin)
 
         if value is None:
             value = default
@@ -192,7 +198,7 @@ class XlConfigParser(SafeConfigParser):
         if value == "false" or value == "False": value = False
         
         self.set(*self.get_section_keyv(key, plugin, str(value)))
-    
+        self.dirty = True
     
     def set_str(self, key, value, plugin=None):
         """
@@ -201,37 +207,42 @@ class XlConfigParser(SafeConfigParser):
         value_str = str(value)
         value_str = re.sub(r'%[^rdf%]?', r'%%', value_str)
         self.set(*self.get_section_keyv(key, plugin, value_str.replace('\n', r'\n')))
-    
+        self.dirty = True
     
     def set_int(self, key, value, plugin=None):
         """
             Sets an int
         """
         self.set(*self.get_section_keyv(key, plugin, str(value)))
-    
+        self.dirty = True
     
     def set_float(self, key, value, plugin=None):
         """
             Sets a float
         """
         self.set(*self.get_section_keyv(key, plugin, str(value)))
-    
+        self.dirty = True
     
     def set_list(self, key, value, plugin=None):
         """
             Sets a list
         """
         self.set(*self.get_section_keyv(key, plugin, str(value)))
-
+        self.dirty = True
     
     def save(self): 
         """
             Saves the settings
         """
+        if self.writing: return True
+        if not self.dirty: return True
+        self.writing = True
+        self.dirty = False
         f = open(self.loc, 'w')
         self.write(f)
         f.close()
-
+        self.writing = False
+        return True
     
     def optionxform(self, option):
         return str(option)
@@ -249,12 +260,16 @@ class Config:
         self.config = XlConfigParser(loc)
         self.loc = loc
 
+        # save settings every 30 seconds
+        gobject.timeout_add(30000, self.save)
+
     def get_crypted(self, key, default="", plugin=None):
         """
             Gets a string from the config file and decrypts it
         """
         string = self.config.get_str(key, default, plugin)
-        
+        if not string: return default
+
         # convert it from hex
         new = ''
         vals = string.split()
@@ -348,7 +363,7 @@ class Config:
 
         for plug in plug_sections:
             plugin = plug.split('/')[1]
-            plugins[plugin] = self.get_boolean("enabled", False, plugin=plugin)
+            plugins[plugin] = self.get_boolean("enabled", True, plugin=plugin)
 
         return plugins
     
@@ -398,6 +413,7 @@ class Config:
             Saves the settings
         """
         self.config.save()
+        return True
     
 
     def __setitem__(self, key, value): 
@@ -436,11 +452,7 @@ class Config:
 def _is_list(s):
     return s and s[0] == '[' and s[-1] == ']'
 def _is_int(s):
-    try:
-        int(s)
-        return True
-    except ValueError:
-        return False
+    return s.isdigit()
 def _is_float(s):
     try:
         float(s)
