@@ -16,6 +16,7 @@
 
 import os, re, urllib
 from gettext import gettext as _, ngettext
+import operator
 import pygtk
 pygtk.require('2.0')
 import gtk, pango
@@ -30,25 +31,32 @@ def create_rating_images(caller):
     """
     if (caller.rating_width != caller.old_r_w and caller.rating_width != 0):
         caller.rating_images = []
-        star_size = caller.rating_width / 4
+        star_size = caller.rating_width / 5
 
         star = gtk.gdk.pixbuf_new_from_file_at_size(
             xl.path.get_data('images', 'star.png'), star_size, star_size)
 
-        star_size -= 1
-
         full_image = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, caller.rating_width, star_size)
         full_image.fill(0xffffff00) # transparent white
-        for x in range(0, 4):
+        for x in range(0, 5):
             star.copy_area(0, 0, star_size, star_size, full_image, star_size * x, 0)
         caller.rating_images.insert(0, full_image)
-        for x in range(7, 0, -1):
+        for x in range(5, 0, -1):
             this_image = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, caller.rating_width, star_size)
             this_image.fill(0xffffff00) # transparent white
-            full_image.copy_area(0, 0, int(x * star_size / 2.0), star_size, this_image, 0, 0)
+            full_image.copy_area(0, 0, int(x * star_size), star_size, this_image, 0, 0)
             caller.rating_images.insert(0, this_image)
         caller.old_r_w = caller.rating_width
 
+class Column(object):
+    __slots__ = ['id', 'display', 'size']
+    def __init__(self, id, display, size):
+        self.id = id
+        self.display = display
+        self.size = size
+    def __repr__(self):
+        return 'Column(%s, %s, %s)' % (`self.id`, `self.display`, `self.size`)
+
 class TracksListCtrl(gtk.VBox):
     """
         Represents the track/playlist table
@@ -58,39 +66,31 @@ class TracksListCtrl(gtk.VBox):
     old_r_w = -1
     row_height = 16
     
-    default_columns = ('#', _('Title'), _('Album'), _('Artist'), _('Length'))
-    col_items = ["#",
-        _("Title"), _("Artist"), _("Album"), _("Length"), _("Disc"),
-        _("Rating"), _("Year"), _("Genre"), _("Bitrate"), _("Location"),
-        _("Filename")]
-    col_map = {
-        '#': 'track',
-        _('Title'): 'title',
-        _('Artist'): 'artist',
-        _('Album'): 'album',
-        _('Length'): 'length',
-        _('Disc'): 'disc_id',
-        _('Rating'): 'rating',
-        _('Year'): 'year',
-        _('Genre'): 'genre',
-        _('Bitrate'): 'bitrate',
-        _('Location'): 'io_loc',
-        _('Filename'): 'filename'
-        }
-    size_map = {
-        '#': 30,
-        _('Title'): 200,
-        _('Artist'): 150,
-        _('Album'): 150,
-        _('Length'): 50,
-        _('Disc'): 30,
-        _('Rating'): 80,
-        _('Year'): 50,
-        _('Genre'): 100,
-        _('Bitrate'): 30,
-        _('Location'): 100,
-        _('Filename'): 50
-    }
+    COLUMNS = [
+        Column('track', _('#'), 30),
+        Column('title', _('Title'), 200),
+        Column('artist', _('Artist'), 150),
+        Column('album', _('Album'), 150),
+        Column('length', _('Length'), 50),
+        Column('disc_id', _('Disc'), 30),
+        Column('rating', _('Rating'), 64),
+        Column('year', _('Year'), 50),
+        Column('genre', _('Genre'), 100),
+        Column('bitrate', _('Bitrate'), 30),
+        Column('io_loc', _('Location'), 200),
+        Column('filename', _('Filename'), 200),
+        Column('playcount', _('Playcount'), 50),
+    ]
+
+    COLUMN_IDS = []
+    column_by_id = {}
+    column_by_display = {}
+    for col in COLUMNS:
+        COLUMN_IDS.append(col.id)
+        column_by_id[col.id] = col
+        column_by_display[col.display] = col
+
+    default_column_ids = ['track', 'title', 'album', 'artist', 'length']
 
     prep = "track"
     type = 'track'
@@ -105,7 +105,7 @@ class TracksListCtrl(gtk.VBox):
         self.exaile = exaile
         self.list = xlmisc.DragTreeView(self)
         self.list.set_rules_hint(True)
-        self.list.set_enable_search(False)
+        self.list.set_enable_search(True)
         self.songs = library.TrackData()
 
         self.scroll = gtk.ScrolledWindow()
@@ -131,19 +131,27 @@ class TracksListCtrl(gtk.VBox):
         self.tpm = None
         self.plugins_item = None
         self.setup_columns()
-
+        self.setup_events()
+        
         create_rating_images(self)
 
         self.show()
-        
-        self.setup_events()
-
 
     def close_page(self):
         """
             Called when this page in the notebook is closed
         """
-        pass
+        #Save the playlist order to the db
+        cur = self.db.cursor()
+        sort_index = 0;
+        for track in self.songs:
+            if track.type == 'stream': continue
+            path_id = library.get_column_id(self.db, 'paths', 'name', track.loc)
+            self.db.execute("UPDATE playlist_items SET sort_index=? " \
+            "WHERE playlist_items.path=?" 
+                            , (sort_index,path_id) )
+            sort_index = sort_index + 1
+        self.db.commit()
 
     def drag_data_received(self, tv, context, x, y, selection, info, etime):
         """
@@ -154,7 +162,6 @@ class TracksListCtrl(gtk.VBox):
             self.list.targets,
             gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE)
 
-        model = tv.get_model()
         loc = list(selection.get_uris())
         counter = 0
  
@@ -170,6 +177,7 @@ class TracksListCtrl(gtk.VBox):
             if os.path.isdir(l.replace("file://", "")):
                 # in this case, it is a folder
                 for root, dirs, files in os.walk(l.replace("file://", '')):
+                    files.sort()
                     for file in files:
                         (stuff, ext) = os.path.splitext(file)
                         if ext.lower() in media.SUPPORTED_MEDIA:
@@ -178,7 +186,7 @@ class TracksListCtrl(gtk.VBox):
         drop_info = tv.get_dest_row_at_pos(x, y)
         if drop_info:
             path, position = drop_info
-            iter = model.get_iter(path)
+            iter = self.model.get_iter(path)
             if (position == gtk.TREE_VIEW_DROP_BEFORE or
                 position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
                 first = False
@@ -197,7 +205,22 @@ class TracksListCtrl(gtk.VBox):
                     # check plugins
                     song = self.get_plugin_track(l)
 
-            if not song or song in self.songs: continue
+            if not song: continue
+            if song in self.songs:
+                # If the song is in drag_delete, we should remove it now
+                # so it gets added properly in its new position.  Otherwise,
+                # we are adding a track to the playlist that is already there,
+                # and should do nothing.
+                if song in self.drag_delete:
+                    index = self.songs.index(song)
+                    itera = self.model.get_iter((index,))
+                    self.model.remove(itera)
+                    self.songs.remove(song)
+                    self.drag_delete.remove(song)
+                    self.update_songs()
+                else:
+                    continue
+
 
             if not drop_info:
                 # if there's no drop info, just add it to the end
@@ -206,18 +229,51 @@ class TracksListCtrl(gtk.VBox):
                 if not first:
                     first = True
                     ar = self.get_ar(song)
-                    iter = self.model.insert_before(iter, ar)
+                    if self.model.iter_is_valid(iter):
+                        iter = self.model.insert_before(iter, ar)
+                    else:
+                        iter = self.model.append(ar)
                 else:
                     ar = self.get_ar(song)
-                    iter = self.model.insert_after(iter, ar)
+                    path = self.model.get_path(iter)
+                    if self.model.iter_is_valid(iter):
+                        iter = self.model.insert_after(iter, ar)
+                    else:
+                        iter = self.model.append(ar)
+
                 if counter >= 20:
                     xlmisc.finish()
                     counter = 0
                 else:
                     counter += 1
-            if not song in self.playlist_songs:
+                    
+                # Update here in case multiple tracks are being dragged.
+                self.update_songs()
+                
+            # For some reason songs not in the database will always
+            # show up as not in playlist_songs even if it is already there.
+            # So instead of just appending if song is in self.playlist_songs, 
+            # we compare track.loc to song.loc.  If that's there, we assign
+            # song to the already existing track, otherwise, we append the
+            # new song like normal.
+            found_track = False
+            for track in self.playlist_songs:                
+                if track.loc == song.loc:
+                    song = track
+                    found_track = True
+            if not found_track:
                 self.playlist_songs.append(song)
 
+        # Now that we've inserted new rows for the dragged tracks
+        # we need to delete the original rows and track entries.
+        if context.action == gtk.gdk.ACTION_MOVE:
+            for track in self.drag_delete:
+                index = self.songs.index(track)
+                iter = self.model.get_iter((index,))
+                self.model.remove(iter)
+                self.songs.remove(track)
+            self.drag_delete = []
+
         if context.action == gtk.gdk.ACTION_MOVE:
             context.finish(True, True, etime)
         else:
@@ -260,6 +316,7 @@ class TracksListCtrl(gtk.VBox):
         index = self.songs.index(track)
         path = (index,)
         self.list.scroll_to_cell(path)
+        self.list.set_cursor(path)
     
     def get_songs(self):
         """
@@ -352,13 +409,11 @@ class TracksListCtrl(gtk.VBox):
                 loc.append(urllib.quote(str(song.loc)))
             delete.append(song)
             
-        if context.action == gtk.gdk.ACTION_MOVE:
-            for song in delete:
-                index = self.songs.index(song)
-                iter = self.model.get_iter((index,))
-                self.model.remove(iter)
-                self.songs.remove(song)
-
+        # We can't remove the tracks in delete until the drag operation is
+        # complete, because doing so causes strange dragging behavior.
+        # So we just make the list available to the drag_data_recieved 
+        # function to make use of.
+        self.drag_delete = delete
         selection.set_uris(loc)
 
     def get_iter(self, song):
@@ -379,7 +434,7 @@ class TracksListCtrl(gtk.VBox):
         """
             Called when someone presses a key
         """
-
+        
         selection = self.list.get_selection()
         (model, pathlist) = selection.get_selected_rows()
 
@@ -402,15 +457,6 @@ class TracksListCtrl(gtk.VBox):
                 path = pathlist[0]
                 if path[0] >= len(self.songs): path = (path[0] - 1,)
                 selection.select_path(path)
-        # Q
-        elif event.keyval == gtk.keysyms.q and self.type != 'queue':
-            for path in pathlist:
-                iter = model.get_iter(path)
-                track = model.get_value(iter, 0)
-                if not track in self.exaile.player.queued:
-                    self.exaile.player.queued.append(track)
-                else:
-                    self.exaile.player.queued.remove(track)
 
             update_queued(self.exaile)
             self.queue_draw()
@@ -439,33 +485,43 @@ class TracksListCtrl(gtk.VBox):
 
         self._col_count = 0
         self._length_id = -1
-        cols = self.exaile.settings.get_list("ui/col_order", self.col_items)
-        for col in self.col_items:
-            if not col in cols:
-                cols.append(col)
 
-        self.append_map = [self.col_map[col] for col in cols if col]
+        col_ids = self.exaile.settings.get_list("ui/col_order", None)
+        search_column = self.exaile.settings.get_str("ui/search_column","Title")
+        
+        cols = None
+        if col_ids:
+            cols = []
+            for col_id in col_ids[:]:
+                col = self.column_by_id.get(col_id)
+                if col: # Good entries only
+                    cols.append(col)
+                else:
+                    col_ids.remove(col_id)
+        if cols:
+            for col in self.COLUMNS:
+                if not col in cols:
+                    cols.append(col)
+                    col_ids.append(col.id)
+        else:
+            cols = self.COLUMNS[:]
+            col_ids = self.COLUMN_IDS[:]
 
-        self.setup_model(self.append_map)
+        self.append_map = col_ids
+        self.setup_model(col_ids)
 
         count = 3
         first_col = True
-        columns_settings = self.exaile.settings.get_list("ui/%s_columns" % (self.prep,))
+        columns_settings = self.exaile.settings.get_list("ui/%s_columns" % self.prep)
 
-        for name in cols:
-            if not self.size_map.has_key(name): continue
+        for col_struct in cols:
             # get cell renderer
             cellr = gtk.CellRendererText()
-            if _(name) == _("Rating"):
+            if col_struct.id == 'rating':
                 cellr = gtk.CellRendererPixbuf()
                 cellr.set_property("follow-state", False)
-            mapval = self.col_map
-            
-            show = False
-            if name in columns_settings:
-                show = True
 
-            if show:
+            if col_struct.id in columns_settings:
                 if first_col:
                     first_col = False
                     pb = gtk.CellRendererPixbuf()
@@ -473,7 +529,7 @@ class TracksListCtrl(gtk.VBox):
                     pb.set_property('xalign', 0.0)
                     stop_pb = gtk.CellRendererPixbuf()
                     stop_pb.set_fixed_size(12, 12)
-                    col = gtk.TreeViewColumn(_(name))
+                    col = gtk.TreeViewColumn(col_struct.display)
                     col.pack_start(pb, False)
                     col.pack_start(stop_pb, False)
                     col.pack_start(cellr, True)
@@ -483,21 +539,23 @@ class TracksListCtrl(gtk.VBox):
                     col.set_cell_data_func(pb, self.icon_data_func)
                     col.set_cell_data_func(stop_pb, self.stop_icon_data_func)
                 else:
-                    col = gtk.TreeViewColumn(_(name), cellr, text=count)
-                
-                if _(name) == _("Length"):
+                    col = gtk.TreeViewColumn(col_struct.display, cellr, text=count)
+
+                if col_struct.id == 'length':
                     col.set_cell_data_func(cellr, self.length_data_func)
-                elif _(name) == "#":
+                elif col_struct.id == 'track':
                     col.set_cell_data_func(cellr, self.track_data_func)
-                elif _(name) == _('Disc'):
+                elif col_struct.id == 'disc_id':
                     col.set_cell_data_func(cellr, self.disc_data_func)
-                elif _(name) == _("Rating"):
+                elif col_struct.id == 'rating':
                     col.set_attributes(cellr, pixbuf=1)
                     col.set_cell_data_func(cellr, self.rating_data_func)
+                else:
+                    col.set_cell_data_func(cellr, self.default_data_func)
 
-                setting_name = "ui/%scol_width_%s" % (self.prep, name)
+                setting_name = "ui/%scol_width_%s" % (self.prep, col_struct.id)
                 width = self.exaile.settings.get_int(setting_name, 
-                    self.size_map[name])
+                    col_struct.size)
                 col.set_fixed_width(width)
 
                 resizable = self.exaile.settings.get_boolean('ui/resizable_cols',
@@ -512,9 +570,12 @@ class TracksListCtrl(gtk.VBox):
                     col.set_sort_indicator(False)
 
                 if not resizable:
-                    if _(name) in (_("Title"), _("Artist"), _("Album"), _("Location")):
-                        col.set_expand(True)
-                        col.set_fixed_width(1)
+                    if col_struct.id in ('title', 'artist', 'album', 'io_loc', 'genre'):
+                        if col_struct.id != 'genre': 
+                            col.set_expand(True)
+                            col.set_fixed_width(1)
+                        else:
+                            col.set_fixed_width(80)
                         col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
                         cellr.set_property('ellipsize', pango.ELLIPSIZE_END)
                     else:
@@ -523,23 +584,26 @@ class TracksListCtrl(gtk.VBox):
                     col.set_resizable(True)
                     col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
-                if _(name) == '#':
+                if col_struct.id in ('track', 'playcount'):
                     cellr.set_property('xalign', 1.0)
-
-                col.set_widget(gtk.Label(_(name)))
+                    
+                # Update which column to search for when columns are changed
+                if col_struct.display == search_column:
+                    self.list.set_search_column(count)
+                col.set_widget(gtk.Label(col_struct.display))
                 col.get_widget().show()
                 self.list.append_column(col)
                 col.get_widget().get_ancestor(gtk.Button).connect('button_press_event', self.press_header)
             count = count + 1
         self.changed_id = self.list.connect('columns-changed', self.column_changed)
-
+    
     def press_header(self, widget, event):
         if event.button != 3:
             return False
         menu = self.exaile.xml.get_widget('columns_menu_menu')
         menu.popup(None, None, None, event.button, event.time)
         return True
-			
+
     def header_toggle(self, menuitem, column):
         column.set_visible(not column.get_visible())
 
@@ -550,7 +614,7 @@ class TracksListCtrl(gtk.VBox):
         self.list.disconnect(self.changed_id)
         cols = []
         for col in self.list.get_columns():
-            cols.append(col.get_title())
+            cols.append(self.column_by_display[col.get_title()].id)
             self.list.remove_column(col)
 
         self.exaile.settings['ui/col_order'] = cols
@@ -560,15 +624,72 @@ class TracksListCtrl(gtk.VBox):
         """
             Called when the user resizes a column
         """
-        name = "ui/%scol_width_%s" % (self.prep, col.get_title())
+        col_struct = self.column_by_display[col.get_title()]
+        name = "ui/%scol_width_%s" % (self.prep, col_struct.id)
         self.exaile.settings[name] = col.get_width()
-        if col.get_title() == _("Rating"):
+        if col_struct.id == 'rating':
             self.rating_width = min(col.get_width(), self.row_height * 4)
             create_rating_images(self)
 
+    def icon_data_func(self, col, cell, model, iter):
+        """
+            Sets track status (playing/paused/queued) icon
+        """
+
+        item = model.get_value(iter, 0)
+        image = None
+
+        if item == self.exaile.player.current:
+            if self.exaile.player.is_playing():
+                image = self.playimg
+            elif self.exaile.player.is_paused():
+                image = self.pauseimg
+        elif item in self.exaile.player.queued:
+            index = self.exaile.player.queued.index(item)
+            image = xlmisc.get_text_icon(self.exaile.window,
+                str(index + 1), 18, 18)
+
+        cell.set_property('pixbuf', image)
+
+    def stop_icon_data_func(self, col, cell, model, iter):
+        """
+            Sets "stop after this" icon
+        """
+
+        item = model.get_value(iter, 0)
+        image = None
+        
+        if item == self.exaile.player.stop_track:
+            image = self.exaile.window.render_icon('gtk-stop', 
+                gtk.ICON_SIZE_MENU) 
+            image = image.scale_simple(12, 12, gtk.gdk.INTERP_BILINEAR)
+        
+        cell.set_property('pixbuf', image)  
+
+    def rating_data_func(self, col, cell, model, iter):
+        item = model.get_value(iter, 0)
+        if not item._rating: return
+        try:
+            idx = item._rating - 1
+            cell.set_property('pixbuf', self.rating_images[idx])
+        except IndexError:
+            if idx > 5: idx = 5
+            elif idx < 0: idx = 0
+
+    def set_cell_weight(self, cell, item):
+        """
+            Sets a CellRendererText's "weight" property according to whether
+            `item` is the currently playing track.
+        """
+        if item == self.exaile.player.current:
+            weight = pango.WEIGHT_HEAVY
+        else:
+            weight = pango.WEIGHT_NORMAL
+        cell.set_property('weight', weight)
+
     def disc_data_func(self, col, cell, model, iter):
         """
-            formats the disc
+            Formats the disc
         """
         item = model.get_value(iter, 0)
         if item.disc_id is None or item.disc_id == -1 or \
@@ -576,6 +697,7 @@ class TracksListCtrl(gtk.VBox):
             cell.set_property('text', '')
         else:
             cell.set_property('text', item.disc_id)
+        self.set_cell_weight(cell, item)
 
     def filename_data_func(self, col, cell, model, iter):
         """
@@ -583,6 +705,24 @@ class TracksListCtrl(gtk.VBox):
         """
         item = model.get_value(iter, 0)
         cell.set_property('text', os.path.basename(item.io_loc))
+        self.set_cell_weight(cell, item)
+
+    def length_data_func(self, col, cell, model, iter):
+        """ 
+            Formats the track length
+        """
+        item = model.get_value(iter, 0)
+        if item.type == 'podcast':
+            text = item.length
+            if item.length == "0:00":
+                text = "N/A"
+        elif item.type == 'stream':
+            text = ''
+        else:
+            seconds = item.duration
+            text = "%s:%02d" % (seconds / 60, seconds % 60)
+        cell.set_property('text', text)
+        self.set_cell_weight(cell, item)
 
     def track_data_func(self, col, cell, model, iter):
         """
@@ -594,6 +734,14 @@ class TracksListCtrl(gtk.VBox):
             cell.set_property('text', '')
         else:
             cell.set_property('text', item.track)
+        self.set_cell_weight(cell, item)
+
+    def default_data_func(self, col, cell, model, iter):
+        """
+            For use in CellRendererTexts that don't have special data funcs.
+        """
+        item = model.get_value(iter, 0)
+        self.set_cell_weight(cell, item)
 
     # sort functions courtesy of listen (http://listengnome.free.fr), which
     # are in turn, courtesy of quodlibet.  
@@ -604,7 +752,7 @@ class TracksListCtrl(gtk.VBox):
         title = column.get_title()
         count = 0
         for col in self.list.get_columns():
-            if column.get_title() == col.get_title():
+            if title == col.get_title():
                 order = column.get_sort_order()
                 if (not column.get_sort_indicator() or 
                     order == gtk.SORT_DESCENDING):
@@ -633,8 +781,7 @@ class TracksListCtrl(gtk.VBox):
         if attr in ('album', 'title'):
             get_key = lambda track: spec_strip(getattr(track, attr).lower())
         elif attr == 'artist':
-            # the_strip(track.artist.lower()) is the same as the next sort key
-            get_key = lambda track: None
+            get_key = lambda track: the_strip(track.artist.lower())
         elif attr == 'length':
             get_key = lambda track: track.get_duration()
         else:
@@ -642,12 +789,11 @@ class TracksListCtrl(gtk.VBox):
 
         s = [
             (get_key(track),
-            the_strip(track.artist.lower()),
-            spec_strip(track.album.lower()),
-            track.track,
             track)
             for track in songs]
-        s.sort(reverse=reverse)
+
+        s  = sorted(s, key=operator.itemgetter(0), reverse=reverse)
+
         songs = [track[-1] for track in s]
         return songs
 
@@ -657,69 +803,10 @@ class TracksListCtrl(gtk.VBox):
         """
         for col in self.list.get_columns():
             if col.get_sort_indicator():
-                return (self.col_map[col.get_title()], 
+                return (self.column_by_display[col.get_title()].id,
                     col.get_sort_order() == gtk.SORT_DESCENDING)
         return 'album', False
 
-    def stop_icon_data_func(self, col, cellr, model, iter):
-        """
-            sets stop icon
-        """
-
-        item = model.get_value(iter, 0)
-        image = None
-        
-        if item == self.exaile.player.stop_track:
-            image = self.exaile.window.render_icon('gtk-stop', 
-                gtk.ICON_SIZE_MENU) 
-            image = image.scale_simple(12, 12, gtk.gdk.INTERP_BILINEAR)
-        
-        cellr.set_property('pixbuf', image)  
-            
-    def icon_data_func(self, col, cellr, model, iter):
-        """
-            sets track icon
-        """
-
-        item = model.get_value(iter, 0)
-        image = None
-
-        if item == self.exaile.player.current:
-            if self.exaile.player.is_playing():
-                image = self.playimg
-            elif self.exaile.player.is_paused():
-                image = self.pauseimg
-        elif item in self.exaile.player.queued:
-            index = self.exaile.player.queued.index(item)
-            image = xlmisc.get_text_icon(self.exaile.window,
-                str(index + 1), 18, 18)
-
-        cellr.set_property('pixbuf', image)
-
-    def rating_data_func(self, col, cellr, model, iter):
-        item = model.get_value(iter, 0)
-        if not item.rating: return
-        idx = len(item.rating) / 2 - 1
-        cellr.set_property('pixbuf', self.rating_images[idx])
-
-    def length_data_func(self, col, cellr, model, iter):
-        """ 
-            Formats the track length
-        """
-        item = model.get_value(iter, 0)
-
-        if item == 'podcast':
-            cellr.set_property('text', item.length)
-            return
-
-        seconds = item.duration
-        text = "%s:%02d" % (seconds / 60, seconds % 60)
-
-        if item.type == 'stream':
-            text = ''
-
-        cellr.set_property('text', text)
-
     def update_col_settings(self):
         """
             Updates the settings for a specific column
@@ -738,7 +825,7 @@ class TracksListCtrl(gtk.VBox):
             Sets the songs in this table (expects a list of tracks)
         """
         self.songs = library.TrackData()
-
+        search_column = self.exaile.settings.get_str("ui/search_column","Title")
         # save sort indicators, because they get reset when you set the model
         indicators = {}
         self.model.clear()
@@ -748,11 +835,17 @@ class TracksListCtrl(gtk.VBox):
         self.list.set_model(self.model_blank)
         if update_playlist: self.playlist_songs = songs
         for song in songs:
-            self.append_song(song)
+            self.append_song(song,False)
+        self.songs = songs;
 
         self.list.set_model(self.model)
+        count = 3
         for col in self.list.get_columns():
             col.set_sort_indicator(indicators[col.get_title()])
+            # We have to set search_column here because the model was cleared
+            if col.get_title() == search_column:
+                self.list.set_search_column(count)
+            count = count + 1  
 
     def get_ar(self, song):
         """
@@ -763,14 +856,14 @@ class TracksListCtrl(gtk.VBox):
             ar.append(getattr(song, field))
         return ar
 
-    def append_song(self, song):
+    def append_song(self, song, append_to_list = True):
         """
             Adds a song to this view
         """
         ar = self.get_ar(song)
 
         self.model.append(ar)
-        if not song in self.songs: self.songs.append(song)
+        if append_to_list: self.songs.append(song)
 
     def update_iter(self, iter, song):
         """
@@ -779,6 +872,13 @@ class TracksListCtrl(gtk.VBox):
         ar = self.get_ar(song)
         self.model.insert_after(iter, ar)
         self.model.remove(iter)
+    
+    def update_song_list(self, song, orig_song):
+        """
+            Updates the in the song list with the new Track object
+        """
+        index = self.songs.index(orig_song)
+        self.songs[index] = song
 
     def refresh_row(self, song):
         """
@@ -793,10 +893,11 @@ class TracksListCtrl(gtk.VBox):
             if not check: break
             if check == song or check.io_loc == song.io_loc:
                 self.update_iter(iter, song)
+                self.update_song_list(song, check)
                 break
             iter = self.model.iter_next(iter)
             if not iter: break
-       
+      
         if not paths: return
         for path in paths:
             selection.select_path(path)
@@ -814,6 +915,7 @@ class TracksListCtrl(gtk.VBox):
         self.tpm = tpm        
                 
         songs = self.get_selected_tracks()
+        t = songs[0].type
 
         n_selected = len(songs)
 
@@ -832,15 +934,21 @@ class TracksListCtrl(gtk.VBox):
                 self.exaile.on_stop_track, 'gtk-stop')
         tpm.append_separator()
 
+        if t == 'podcast':
+            tpm.append(_('Download Podcasts'), self.download_podcast,
+            'gtk-save')
         tpm.append(_("Edit Track Information"), lambda e, f:
             editor.TrackEditor(self.exaile, self), 'gtk-edit')
 
         rm = xlmisc.Menu()
-        self.rating_ids = []
 
-        for i in range(0, 8):
-            item = rm.append_image(self.rating_images[i],
-                lambda w, e, i=i: editor.update_rating(self, i))
+        for i in range(0, 5):
+            if i == 0:
+                item = rm.append('-', lambda w, e, i=i:
+                    editor.update_rating(self, i))
+            else:
+                item = rm.append_image(self.rating_images[i],
+                    lambda w, e, i=i: editor.update_rating(self, i))
 
         star_icon = gtk.gdk.pixbuf_new_from_file_at_size(
             xl.path.get_data('images', 'star.png'), 16, 16)
@@ -851,7 +959,6 @@ class TracksListCtrl(gtk.VBox):
 
         tpm.append_menu(_("Set Track Rating"), rm, 'exaile-star-icon')
 
-        t = songs[0].type
         if t == 'file':
             bm = xlmisc.Menu()
             bm.append(ngettext("Burn Selected Track", "Burn Selected Tracks",
@@ -896,6 +1003,7 @@ class TracksListCtrl(gtk.VBox):
                 tpm.append_menu(_("Add to Saved Stations"),
                     self.playlists_menu, 'gtk-add')
 
+
         if n_selected == 1:
             info = tpm.append(_("Information"), self.get_track_information,
                 'gtk-info')
@@ -918,8 +1026,15 @@ class TracksListCtrl(gtk.VBox):
             tpm.append(_("Show in Collection"), self.show_in_collection,
                 'exaile-track-icon')
 
-        tpm.append(_("Remove from Current"),
+        rm = xlmisc.Menu()
+        rm.append(_("Remove Selected from Playlist"),
             self.delete_tracks, 'gtk-delete')
+        rm.append(_("Blacklist Selected"),
+            self.delete_tracks, 'gtk-delete', 'blacklist')
+        rm.append(_("Delete Selected"),
+            self.delete_tracks, 'gtk-delete', 'delete')
+
+        tpm.append_menu(_("Remove"), rm, 'gtk-delete')
 
         # plugins menu items
         if self.exaile.plugins_menu.get_children():
@@ -970,7 +1085,19 @@ class TracksListCtrl(gtk.VBox):
             The popup menu that is displayed when you right click in the
             playlist
         """
-        if not event.button == 3: return
+        if not event.button == 3: 
+            try: path, col, cellx, celly = self.list.get_path_at_pos(int(event.x),
+            int(event.y))
+            except: 
+                xlmisc.log_exception()
+                return True
+
+            if col.get_title() == _('Rating'):
+                count = int(float(cellx + 5) / (float(self.rating_width / 5)))
+                print count
+            return True
+            print path, col, cellx, celly, col.get_title()
+            
         selection = self.list.get_selection()
         self.setup_tracks_menu()
 
@@ -1028,6 +1155,15 @@ class TracksListCtrl(gtk.VBox):
         except IOError:
             xlmisc.log('Could not save station list')
 
+    def download_podcast(self, event, type):
+        """
+            Downloads the selected podcasts
+        """
+        songs = self.get_selected_tracks()
+        for song in songs:
+            if song.type == 'podcast':
+                self.exaile.pradio_panel.add_podcast_to_queue(song)
+
     def delete_tracks(self, event, type): 
         """
             Deletes tracks, or just removes them from the playlist
