@@ -14,8 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-import md5, os, random, re, threading, time, traceback
-from gettext import gettext as _
+import md5, os, random, re, threading, time, traceback, gc, sys
+from gettext import gettext as _, ngettext
 import gobject, gtk
 from xl import common, media, db, audioscrobbler, xlmisc, dbusinterface
 from xl.db import DBOperationalError
@@ -65,22 +65,20 @@ def get_suggested_songs(exaile, db, song, s, count, done_func):
         xlmisc.log("Fetching suggested tracks for %s" % song.artist)
         try:
             lastfm = audioscrobbler.AudioScrobblerQuery(artist=song.artist)
-            artists = []
-            for artist in lastfm.similar():
-                artists.append(artist.name)
-
-            random.shuffle(artists)
-            all.extend(artists)
-           
+            artists = [artist.name for artist in lastfm.similar()]
         except audioscrobbler.AudioScrobblerError:
-            pass
+            continue
         except:
             xlmisc.log_exception()
+            continue
+
+        random.shuffle(artists)
+        all.extend(artists)
         if len(all) >= 100: break
 
     gobject.idle_add(done_func, all, count)    
 
-class TrackData(list):
+class TrackData:
     """
         Represents a list of tracks
     """
@@ -88,18 +86,39 @@ class TrackData(list):
         """
             Initializes the list
         """
-        self.paths = dict()
+        self.total_length = 0
+        self.paths = {}
+        self._inner = []
         if tracks:
             for track in tracks:
                 self.append(track)
 
+    def __getitem__(self, index):
+        return self._inner[index]
+
+    def __setitem__(self, index, value):
+        old = self._inner[index]
+        try:
+            del self.paths[old.loc]
+        except KeyError:
+            pass
+        self.paths[value.loc] = value
+        self._inner[index] = value
+ 
+    def __len__(self):
+        return len(self._inner)
+
+    def index(self, item):
+        return self._inner.index(item)
+ 
     def append(self, track):
         """
             Adds a track to the list
         """
         if not track: return
         self.paths[track.loc] = track
-        list.append(self, track)
+        self._inner.append(track)
+        self.update_total_length(track.get_duration(), appending=True)
 
     def remove(self, track):
         """
@@ -107,23 +126,44 @@ class TrackData(list):
         """
         if not track: return
         try:
-            if not self.paths[track.loc]: return
+            del self.paths[track.loc]
         except KeyError:
             return
-        del self.paths[track.loc]
-        list.remove(self, track)
+        else:
+            self._inner.remove(track)
+        self.update_total_length(track.get_duration(), appending=False)
+    
+    def update_total_length(self, track_duration, appending):
+        if appending:
+            self.total_length += track_duration
+        else:
+            self.total_length -= track_duration
+            
+    def get_total_length(self):
+        """ 
+            Returns length of all tracks in the table as preformatted string
+        """
+        s = self.total_length
+        m, s = divmod(s, 60)
+        h, m = divmod(m, 60)
+        d, h = divmod(h, 24)
+
+        text = []
+        if d: text.append(ngettext("%d day", "%d days", d) % d)
+        if h: text.append(ngettext("%d hour", "%d hours", h) % h)
+        if m: text.append(ngettext("%d minute", "%d minutes", m) % m)
+        if s: text.append(ngettext("%d second", "%d seconds", s) % s)
+
+        text = ", ".join(text)
+
+        return text
+
 
     def for_path(self, path):
         """
-            Returns the track associated with the path
+            Returns the track associated with the path, or None
         """
-
-        if self.paths.has_key(path):
-            track = self.paths[path]
-            if track in self:
-                return self.paths[path]
-
-        return None
+        return self.paths.get(path)
 
 def search(exaile, all, keyword=None, custom=True):
     """
@@ -150,10 +190,12 @@ def search_tracks(parent, db, all, keyword=None, playlist=None, w=None):
     items = []
     args = []
     where = ""
+    if keyword and keyword.replace("\"","") == "":
+        keyword = None
     if keyword != None and w:
         regex = re.compile("\s+WHERE\s", re.DOTALL)
         w = regex.sub(" AND ", w)
-        tokens = keyword.lower().split(' ')
+        tokens = keyword.lower().replace("\"","").split(' ')
 
         anditems = []
         check_fields = ('title', 'artists.name', 'albums.name')
@@ -229,7 +271,13 @@ def search_tracks(parent, db, all, keyword=None, playlist=None, w=None):
         common.error(parent, _("Query Error: %s") % str(e))
         raise e
 
-    for row in cur.fetchall():
+    while True:
+        try:
+            row = cur.fetchone()
+            if not row: break
+        except sqlite3.OperationalError:
+            continue
+
         track = all.for_path(row[0])
         if track == None:
             pass
@@ -245,7 +293,7 @@ def search_tracks(parent, db, all, keyword=None, playlist=None, w=None):
 
 def already_added(t, added):
     """
-        Checks to see if an md5 hash of the title, artist, album and path 
+        Checks to see if the title, artist, album and genre 
         has already been added to the list of tracks
     """
 
@@ -253,7 +301,9 @@ def already_added(t, added):
     if not t.title: t.title = ""
     if not t.album: t.album = ""
     if not t.artist: t.artist = ""
-    h = "%s - %s - %s - %s" % (t.title, t.album, t.artist, t.loc)
+    if not t.genre: t.genre = ""
+
+    h = "%s - %s - %s - %s" % (t.title, t.album, t.artist, t.genre)
 
     if added.has_key(h): return True
     added[h] = 1
@@ -288,7 +338,9 @@ def load_tracks(db, current=None):
             user_rating, 
             rating,
             blacklisted, 
-            time_added
+            time_added,
+            encoding,
+            plays
         FROM tracks, paths, artists, albums 
         WHERE 
             (
@@ -338,7 +390,7 @@ def load_tracks(db, current=None):
     db._close_thread()
     return tracks
 
-def scan_dir(dir, files=None, skip=(), exts=()):
+def scan_dir(dir, files=None, skip=(), exts=(), scanned=None):
     """
         Scans a directory recursively
     """
@@ -348,12 +400,18 @@ def scan_dir(dir, files=None, skip=(), exts=()):
         regex = re.compile(match_string)
     if files is None: 
         files = []
+    if scanned is None:
+        scanned = set()
 
     try:
+        dir = os.path.realpath(dir)
         to_scan = os.listdir(dir)
     except OSError:
         return files
 
+    if dir in scanned: return files
+    scanned.add(dir)
+
     for file in to_scan:
         try:
             file = os.path.join(dir, file)
@@ -365,10 +423,9 @@ def scan_dir(dir, files=None, skip=(), exts=()):
             continue
 
         try:
-            if os.path.isdir(file) and not \
-                os.path.islink(file):
-                    if regex and regex.match(file): continue
-                    scan_dir(file, files=files, skip=skip, exts=exts)
+            if os.path.isdir(file):
+                if regex and regex.match(file): continue
+                scan_dir(file, files=files, skip=skip, exts=exts, scanned=scanned)
         except:
             xlmisc.log("Error scanning %s" % file)
             traceback.print_exc()
@@ -441,7 +498,7 @@ def read_audio_disc(exaile):
 
         minus = info[i + 3] / 75
         tracknum = i + 1
-        song = media.Track("cdda://%d" % tracknum, _("Track %d") % tracknum,
+        song = media.Track("cdda://%d#%s" % (tracknum, device), _("Track %d") % tracknum,
             track=tracknum, length=length)
         song.type = 'cd'
         #if not song.artist: song.artist = _("Unknown Artist")
@@ -534,7 +591,8 @@ def read_track_from_db(db, path, track_type=media.Track):
             rating,
             blacklisted, 
             time_added,
-            encoding
+            encoding,
+            plays
         FROM tracks,paths,artists,albums 
         WHERE 
             (
@@ -576,15 +634,17 @@ def save_track_to_db(db, tr, new=False, prep=''):
             "disc_id": tr.disc_id,
             "genre": tr.genre,
             "track": tr.track,
+            "rating": tr.system_rating,
+            "user_rating": tr._rating,
             "length": tr.duration,
             "bitrate": tr._bitrate,
             "rating": tr.system_rating,
-            "user_rating": tr._rating,
             "blacklisted": tr.blacklisted,
             "year": tr.year,
             "modified": tr.modified,
             "time_added": tr.time_added,
-            "encoding": tr.encoding
+            "encoding": tr.encoding,
+            "plays": tr.playcount
         }, "path=?", (path_id,), new)
 
 class PopulateThread(threading.Thread):
@@ -687,7 +747,12 @@ class PopulateThread(threading.Thread):
 
     def do_function(self, loc):
         db = self.db
-        tr = self.exaile.all_songs.for_path(loc)
+
+        in_library = False
+        tr = self.exaile.all_songs.for_path(unicode(loc,
+            xlmisc.get_default_encoding()))
+        if tr:
+            in_library = True
 
         bl = 0
         if not tr:
@@ -695,12 +760,17 @@ class PopulateThread(threading.Thread):
             if tr and tr.blacklisted: bl = 1
 
         modified = os.stat(loc).st_mtime
+        time_added=''
         if not tr or tr.modified != modified:
             if not tr: new = True
-            else: new = False
+            else: 
+               new = False
+               time_added = tr.time_added
             tr = media.read_from_path(loc)
             tr.blacklisted = bl
             if not tr: return
+            if time_added:
+               tr.time_added=time_added
             tr.modified = modified
             save_track_to_db(db, tr, new)
 
@@ -710,10 +780,9 @@ class PopulateThread(threading.Thread):
                 (path_id,))
         if not tr or tr.blacklisted: return
         
-        if not self.exaile.all_songs.for_path(loc):
+        if not in_library:
             if not already_added(tr, self.added): self.exaile.all_songs.append(tr)
         self.found_tracks.append(tr)
-        already_added(tr, self.added)       
 
     def stop(self):
         """
@@ -729,8 +798,12 @@ class PopulateThread(threading.Thread):
             tracks = None
         gobject.idle_add(self.update_func, num, tracks, 
             self.done_func) 
+        gobject.idle_add(gc.collect)
         PopulateThread.stopped = False
         PopulateThread.running = False
+        self.exaile = None
+        self.directories = None
+        self.added = None
 
 def populate(exaile, db, directories, update_func, delete=True,
     load_tree=False, done_func=None):
@@ -769,6 +842,11 @@ class RemoveTracksThread(PopulateThread):
             db.execute("DELETE FROM tracks WHERE path=?",
                 (path_id,))
 
+        try:
+            self.exaile.all_songs.remove(tr)
+        except ValueError:
+            pass
+
 def remove_tracks(exaile, db, directories, update_func, delete=True,
     load_tree=False, done_func=None):
     """
@@ -796,6 +874,8 @@ class AddTracksThread(PopulateThread):
 
     def do_function(self, loc):
         db = self.db
+
+        if self.exaile.all_songs.for_path(loc): return
         tr = media.read_from_path(loc)
         if not tr: return
         new = True
@@ -804,10 +884,8 @@ class AddTracksThread(PopulateThread):
         path_id = get_column_id(db, 'paths', 'name', unicode(loc, xlmisc.get_default_encoding()))
         db.execute("UPDATE tracks SET included=1 WHERE path=?", (path_id,))
         
-        if not self.exaile.all_songs.for_path(loc):
-            if not already_added(tr, self.added): self.exaile.all_songs.append(tr)
+        if not already_added(tr, self.added): self.exaile.all_songs.append(tr)
         self.found_tracks.append(tr)
-        already_added(tr, self.added)
 
 def add_tracks(exaile, db, directories, update_func, delete=True,
     load_tree=False, done_func=None):
@@ -819,7 +897,6 @@ def add_tracks(exaile, db, directories, update_func, delete=True,
     exaile.thread_pool.append(thread)
     thread.start()
 
-
 class LibraryManager(object):
     def __init__(self, exaile):
         self.exaile = exaile
@@ -905,7 +982,6 @@ class LibraryManager(object):
         self.exaile.collection_panel.songs = self.exaile.all_songs
         self.exaile.collection_panel.track_cache = dict()
 
-
         if not updating:
             xlmisc.log("loading songs")
             gobject.idle_add(self.exaile.playlists_panel.load_playlists)
@@ -943,12 +1019,19 @@ class LibraryManager(object):
             Scans the library
         """
         self.exaile.collection_panel.update_progress(percent)
-        
+
         if percent < 0:
             self.db.db.commit()
             self.db._cursor.close()
             self.db._cursor = self.db.realcursor()
-            self.load_songs(percent==-1)
+
+            # if percent is not -2, it wasn't an auto scan, so we won't reload the
+            # collection tree because that could potentially take a lot of
+            # time if the person has a lot of tracks in their library
+            if percent == -2:
+                self.exaile.collection_panel.songs = self.exaile.all_songs
+                self.exaile.collection_panel.track_cache = dict()
+                gobject.idle_add(self.exaile.collection_panel.load_tree, True)
 
         if done_func:
             done_func(songs)
