@@ -41,6 +41,8 @@ class CollectionPanel(object):
         self.exaile = exaile
         self.db = exaile.db
 
+        self.model = gtk.TreeStore(gtk.gdk.Pixbuf, object, str)
+        self.model_blank = gtk.TreeStore(gtk.gdk.Pixbuf, object, str)
         self.scan_label = None
         self.queue_item = -1
         self.scan_progress = None
@@ -51,6 +53,8 @@ class CollectionPanel(object):
         self.start_count = 0
         self.artist_image = gtk.gdk.pixbuf_new_from_file(xl.path.get_data(
             'images', 'artist.png'))
+        self.year_image = gtk.gdk.pixbuf_new_from_file(xl.path.get_data(
+            'images', 'year.png'))
         self.album_image = self.exaile.window.render_icon('gtk-cdrom',
             gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.track_image = gtk.gdk.pixbuf_new_from_file(xl.path.get_data(
@@ -62,6 +66,7 @@ class CollectionPanel(object):
         self.connect_id = None
         self.setup_widgets()
         playlist.create_rating_images(self)
+        self.cover_cache = {}
 
     def setup_widgets(self):
         """
@@ -152,7 +157,7 @@ class CollectionPanel(object):
             in LoadTree if keyword is not None which will expand the items
             of the tree until the matched node is found
         """
-        self.keyword = self.filter.get_text()
+        self.keyword = unicode(self.filter.get_text(), 'utf-8')
         self.start_count += 1
         self.load_tree()
 
@@ -204,11 +209,14 @@ class CollectionPanel(object):
         em.append_separator()
 
         rm = xlmisc.Menu()
-        self.rating_ids = []
 
-        for i in range(0, 8):
-            item = rm.append_image(self.rating_images[i],
-                lambda w, e, i=i: editor.update_rating(self, i))
+        for i in range(0, 5):
+            if i == 0:
+                item = rm.append('-', lambda w, e, i=i:
+                    editor.update_rating(self, i))
+            else:
+                item = rm.append_image(self.rating_images[i],
+                    lambda w, e, i=i: editor.update_rating(self, i))
 
         em.append_menu(_("Rating"), rm)
         menu.append_menu(ngettext("Edit Track", "Edit Tracks", n_selected), em,
@@ -422,6 +430,9 @@ class CollectionPanel(object):
         if object is None: return
         field = model.get_value(iter, 2)
 
+        if field == 'nofield':
+            cell.set_property('text', object)
+            return
         if hasattr(object, field):
             info = getattr(object, field)
             if not info: info = _('Unknown')
@@ -474,9 +485,7 @@ class CollectionPanel(object):
             xlmisc.log("Clearing tracks cache")
             self.track_cache = dict()
 
-        self.model = gtk.TreeStore(gtk.gdk.Pixbuf, object, str)
-        self.model_blank = gtk.TreeStore(gtk.gdk.Pixbuf, object, str)
-
+        self.model.clear()
         self.tree.set_model(self.model_blank)
         self.root = self.get_initial_root(self.model)
     
@@ -485,6 +494,7 @@ class CollectionPanel(object):
             "artist": self.artist_image,
             "genre": self.genre_image,
             "title": self.track_image,
+            "year": self.year_image,
         }
 
         if self.keyword == "": self.keyword = None
@@ -493,6 +503,10 @@ class CollectionPanel(object):
             ('artist', 'album', 'track', 'title'),
             ('album', 'track', 'title'),
             ('genre', 'artist', 'album', 'track', 'title'),
+            ('genre', 'album', 'artist', 'track', 'title'),
+            ('year', 'artist', 'album', 'track', 'title'),
+            ('year', 'album', 'artist', 'track', 'title'),
+            ('artist', 'year', 'album', 'track', 'title')
         )
         self.order = orders[self.choice.get_active()]
 
@@ -502,6 +516,7 @@ class CollectionPanel(object):
             'genre': 'LSTRIP_SPEC(genre)',
             'title': 'LSTRIP_SPEC(title)',
             'track': 'track', 
+            'year' : 'LSTRIP_SPEC(year)',
         }
         order_by = ', '.join((o_map[o] for o in self.order))
 
@@ -541,7 +556,8 @@ class CollectionPanel(object):
             songs = self.track_cache[key]
         except:
             pass
-        if not songs:
+
+        if not songs or self.name== 'device':
             songs = self.search_tracks(self.keyword, self.all)
             self.track_cache[key] = songs
 
@@ -596,6 +612,7 @@ class CollectionPanel(object):
             Only the very last item of self.order will be a child node
         """
 
+        show_covers = self.exaile.settings.get_boolean('ui/covers_in_collection', False)
         order_nodes = common.idict()
         order = []
         last_songs = []
@@ -651,15 +668,23 @@ class CollectionPanel(object):
                 else:
                     string = "%s - %s" % (string, info)
                     if not string in node_for:
-                        parent = self.model.append(parent, 
-                            [self.image_map[field], track, field])
+                        if field == "album" and show_covers:                                            
+                            album_cover = self.exaile.cover_manager.fetch_cover(track, True)
+                            if not album_cover in self.cover_cache:
+                                self.cover_cache[album_cover] = \
+                                gtk.gdk.pixbuf_new_from_file_at_size(album_cover, 20, 20)
+                            parent = self.model.append(parent, 
+                                [self.cover_cache[album_cover], track, field])
+                        else:                            
+                            parent = self.model.append(parent, 
+                                [self.image_map[field], track, field])
 
                         if info == "track": info = track
                         node_for[string] = parent
                     else: parent = node_for[string]
 
                 if self.keyword and last_parent:
-                    if str(info).lower().find(self.keyword.lower()) > -1:
+                    if self.keyword.lower() in common.to_unicode(info).lower():
                         expanded_paths.append(self.model.get_path(
                             last_parent))
                 last_parent = parent
