@@ -15,7 +15,7 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 import md5, os, re, threading, time, urllib
-from xml.dom import minidom
+from lib import feedparser
 from gettext import gettext as _
 import gobject, gtk
 from xl import common, xlmisc, library, media
@@ -80,7 +80,7 @@ class PodcastQueueThread(threading.Thread):
             if self.stopped: break
             hin = urllib.urlopen(song.loc)
 
-            temp_path = xl.path.get_config('podcasts', 'downloading')
+            temp_path = "%s.part" % download_path
             hout = open(temp_path, "w+")
 
             count = 0
@@ -114,13 +114,16 @@ class PodcastQueueThread(threading.Thread):
             if temp:
                 song.set_len(temp.duration)
                 self.db = self.panel.exaile.db
+                path_id = library.get_column_id(self.db, 'paths', 'name',
+                    song.loc)
+                podcast_path_id = library.get_column_id(self.db, 'paths', 
+                    'name', song.podcast_path)
+
                 self.panel.exaile.db.execute(
-                    "UPDATE podcast_items SET length=%s WHERE podcast_path=%s"
-                    " AND path=%s" % (self.db.p, self.db.p, self.db.p), 
-                    (temp.duration, song.podcast_path, song.loc))
+                    "UPDATE podcast_items SET length=? WHERE podcast_path=?"
+                    " AND path=?", 
+                    (temp.duration, podcast_path_id, path_id))
                 self.panel.exaile.db.commit()
-                if song.podcast_artist:
-                    song.artist = song.podcast_artist
 
             gobject.idle_add(self.transfer_queue.update_song, song)
             xlmisc.log("Downloaded podcast %s" % song.loc)
@@ -293,8 +296,8 @@ class RadioPanel(object):
 
         self.track = gtk.gdk.pixbuf_new_from_file(xl.path.get_data('images',
             'track.png'))
-        self.custom = self.model.append(None, [self.open_folder, "Saved Stations"])
-        self.podcast = self.model.append(None, [self.open_folder, "Podcasts"])
+        self.custom = self.model.append(None, [self.open_folder, _("Saved Stations")])
+        self.podcast = self.model.append(None, [self.open_folder, _("Podcasts")])
 
         # load all saved stations from the database
         rows = self.db.select("SELECT name FROM radio "
@@ -315,8 +318,8 @@ class RadioPanel(object):
         self.tree.expand_row(self.model.get_path(self.custom), False)
         self.tree.expand_row(self.model.get_path(self.podcast), False)
 
-        self.radio_root = self.model.append(None, [self.open_folder, "Radio "
-            "Streams"])
+        self.radio_root = self.model.append(None, [self.open_folder, _("Radio "
+            "Streams")])
 
         self.drivers_expanded = {}
         self.load_nodes = {}
@@ -391,6 +394,7 @@ class RadioPanel(object):
                 else:
                     menu = self.menu
                 menu.popup(None, None, None, event.button, event.time)
+                return True
             else:
                 if object in ("Saved Stations", "Podcasts", "Shoutcast Stations"):
                     return
@@ -405,13 +409,15 @@ class RadioPanel(object):
                 self.open_lastfm(object)
             elif isinstance(object, RadioGenre):
                 if object.driver:
-                    tracks = trackslist.TracksListCtrl(self.exaile)
-                    self.exaile.playlists_nb.append_page(tracks,
-                        xlmisc.NotebookTab(self.exaile, str(object), tracks))
-                    self.exaile.playlists_nb.set_current_page(
-                        self.exaile.playlists_nb.get_n_pages() - 1)
-                    self.exaile.tracks = tracks
-                    object.driver.tracks = tracks
+                    if not hasattr(object.driver, 'no_new_page') or not \
+                        object.driver.no_new_page:
+                        tracks = trackslist.TracksListCtrl(self.exaile)
+                        self.exaile.playlists_nb.append_page(tracks,
+                            xlmisc.NotebookTab(self.exaile, str(object), tracks))
+                        self.exaile.playlists_nb.set_current_page(
+                            self.exaile.playlists_nb.get_n_pages() - 1)
+                        self.exaile.tracks = tracks
+                        object.driver.tracks = tracks
                     object.driver.load_genre(object)
             return True
 
@@ -494,10 +500,13 @@ class RadioPanel(object):
         if not row: return
 
         desc = row[0]
+        limit = self.exaile.settings.get_str('podcast_show_limit', '0')
+        limiter = ""
+        if int(limit) > 0: limiter = "LIMIT %s" % limit
         rows = self.db.select("SELECT paths.name, title, description, length, "
             "pub_date, size FROM podcast_items, paths WHERE podcast_path=? "
-            "AND paths.id=podcast_items.path ORDER BY"
-            " pub_date DESC LIMIT 10", 
+            "AND paths.id=podcast_items.path ORDER BY "
+            "pub_date DESC %s" % limiter, 
             (podcast_path_id,))
 
         songs = library.TrackData()
@@ -507,7 +516,7 @@ class RadioPanel(object):
             info = ({
                 'title': row[1],
                 'artist': row[2],
-                'album': desc,
+                'album': desc.replace('\n', ' '),
                 'loc': row[0],
                 'year': row[4], 
                 'length': row[3], 
@@ -517,33 +526,19 @@ class RadioPanel(object):
             song.set_info(**info)
             song.type = 'podcast'
             song.size = row[5]
+            song.year = song.year.replace(' 00:00:00', '')
 
             (download_path, downloaded) = \
                 self.get_podcast_download_path(row[0])
-            add_item = False
 
-            if not self.exaile.settings.get_boolean('download_feeds', True):
-                add_item = False
-
-                song.download_path = 1
+            if not downloaded:
+                song.download_path = ''
             else:
-                if not downloaded:
+                song.download_path = download_path
 
-                    song.artist = _("Not downloaded")
-                    song.download_path = ''
-                    add_item = True
-                else:
-                    song.download_path = download_path
-
-            song.length = row[3]
             song.podcast_path = wrapper.path
             songs.append(song)
             self.podcasts[song.loc] = song
-            if add_item:
-                song.podcast_artist = row[2] 
-                self.add_podcast_to_queue(song)
-            else:
-                song.podcast_artist = None
 
         self.exaile.new_page(str(wrapper), songs)
 
@@ -571,7 +566,8 @@ class RadioPanel(object):
         """
         (path, ext) = os.path.splitext(loc)
         hash = md5.new(loc).hexdigest()
-        savepath = xl.path.get_config('podcasts')
+        savepath = self.exaile.settings.get_str('feed_download_location',
+            os.path.expanduser('~/.exaile/podcasts')) 
         if not os.path.isdir(savepath):
             os.mkdir(savepath, 0777)
 
@@ -722,33 +718,28 @@ class RadioPanel(object):
             Refreshes a podcast
         """
         try:
-            h = urllib.urlopen(path)
-            xml = h.read()
-            h.close()
+            feed = feedparser.parse(path)
         except IOError:
             gobject.idle_add(common.error, self.exaile.window, 
                 _("Could not read feed."))
             return
 
-        gobject.idle_add(self.parse_podcast_xml, path, item, xml)
+        gobject.idle_add(self.parse_podcast_xml, path, item, feed)
 
-    def parse_podcast_xml(self, path, iter, xml):
+    def parse_podcast_xml(self, path, iter, feed):
         """
             Parses the xml from the podcast and stores the information to the
             database
         """
-        path = str(path)
-        xml = minidom.parseString(xml).documentElement
-        root = xml.getElementsByTagName('channel')[0]
-
-        title = self.get_val(root, 'title')
-        description = self.get_val(root, 'description')
+        title = feed.feed.title
+        description = feed.feed.description
         if not description: description = ""
-        pub_date = self.get_val(root, 'pubDate')
-        print pub_date
-        if not pub_date: pub_date = ""
-        image = self.get_val(root, 'image')
-        if not image: image = ""
+
+        try: pub_date = feed.feed.updated
+        except AttributeError: pub_date = ""
+
+        # TODO: use the image from the feed as the cover image
+        image = ""
         path_id = library.get_column_id(self.db, 'paths', 'name', path)
 
         self.db.execute("UPDATE podcasts SET title=?, "
@@ -757,27 +748,40 @@ class RadioPanel(object):
 
         self.model.set_value(iter, 1, PodcastWrapper(title, path))
         root_title = title
-        items = root.getElementsByTagName('item')
 
-        for item in items:
-            title = self.get_val(item, 'title')
-            link = self.get_val(item, 'link')
+        found_items = []
+        
+        for item in feed['entries']:
+            title = item.title 
+            link = item.link
             print title, link
-            desc = self.get_val(item, 'description')
+            desc = item.description
 
             desc = self.clean_desc(desc)
-            enc = self.get_child(item, 'enclosure')
-            date = self.get_val(item, 'pubDate')
+            enc = None
+            try:
+                enc = item.enclosures
+            except AttributeError:
+                pass
+
+            date = item.updated
             if enc:
-                size = enc[0].getAttribute('length')
-                length = enc[0].getAttribute('duration')
-                loc = str(enc[0].getAttribute("url"))
+                try: size = enc[0].length
+                except AttributeError: size = 0
+                try: length = enc[0].duration
+                except AttributeError: length = 0
+                try: loc = str(enc[0].href)
+                except AttributeError: loc = ''
             else: continue
             loc_id = library.get_column_id(self.db, 'paths', 'name', loc)
+            found_items.append(loc_id)
 
             row = self.db.read_one("podcast_items", "path", 
                 "podcast_path=? AND path=?", (path_id, loc_id))
 
+            # if the feed already exists, don't add it again
+            if row: continue
+
             t = common.strdate_to_time(date)
             date = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(t))
 
@@ -793,6 +797,13 @@ class RadioPanel(object):
                 }, "path=? AND podcast_path=?", 
                 (loc_id, path_id), row == None)
 
+        # delete all items that aren't still in the feed
+        where = ["path!=\"%d\" " % (x) for x in found_items]
+        where = " AND ".join(where)
+        cur = self.db.cursor()
+        cur.execute("DELETE FROM podcast_items WHERE (%s) AND podcast_path=?"
+            % (where,), (path_id,))
+
         self.db.commit()
 
         gobject.timeout_add(500, self.open_podcast, PodcastWrapper(root_title, path))
@@ -810,29 +821,6 @@ class RadioPanel(object):
 
         return desc
 
-    def get_child(self, node, name):
-        """ 
-            Gets a child node
-        """
-        return node.getElementsByTagName(name)
-
-    def get_value(self, node):
-        if hasattr(node, "firstChild") and node.firstChild:
-            return node.firstChild.nodeValue
-        else:
-            return ""
-
-    # the following code stolen from listen
-    def get_val(self, node, name):
-        """
-            Node navigation
-        """
-        node = self.get_child(node, name)
-        if node:
-            return self.get_value(node[0])
-        else:
-            return ""
-            
     def delete_podcast(self, widget, event):
         """ 
             Removes a podcast
@@ -1015,7 +1003,7 @@ class RadioPanel(object):
         if station:
             playlist = station
         else:
-            playlist = item.get_child().get_text()
+            playlist = unicode(item.get_child().get_text(), 'utf-8')
 
         station_id = library.get_column_id(self.db, 'radio', 'name', playlist)
 
