@@ -19,19 +19,14 @@ pygst.require('0.10')
 import gst, gobject, random, time, re, os, md5
 import thread
 from gettext import gettext as _
-from xl import xlmisc, common, media, library
-from xl.gui import playlist as playlistgui
+from xl import xlmisc, common, media, library, logger
+from xl.gui import playlist as playlistgui, information
 import xl.path
-random.seed(time.time())
-
-ASX_REGEX = re.compile(r'href ?= ?([\'"])(.*?)\1', re.DOTALL|re.MULTILINE)
 
 class Player(gobject.GObject):
     """
         This is the main player interface, other engines will subclass it
     """
-    def __init__(self):
-        gobject.GObject.__init__(self)
 
 class GSTPlayer(Player):
     """
@@ -60,30 +55,55 @@ class GSTPlayer(Player):
         """
         self.playbin.set_property('volume', vol)
 
+    def _get_gst_state(self):
+        """
+            Returns the raw GStreamer state
+        """
+        return self.playbin.get_state(timeout=50*gst.MSECOND)[1]
+
+    def get_state(self):
+        """
+            Returns the player state: 'playing', 'paused', or 'stopped'.
+        """
+        state = self._get_gst_state()
+        if state == gst.STATE_PLAYING:
+            return 'playing'
+        elif state == gst.STATE_PAUSED:
+            return 'paused'
+        else:
+            return 'stopped'
+
     def is_playing(self):
         """
             Returns True if the player is currently playing
         """
-        return self.playbin.get_state()[1] == gst.STATE_PLAYING
+        return self._get_gst_state() == gst.STATE_PLAYING
 
     def is_paused(self):
         """
             Returns True if the player is currently paused
         """
-        return self.playbin.get_state()[1] == gst.STATE_PAUSED
+        return self._get_gst_state() == gst.STATE_PAUSED
 
     def on_message(self, bus, message, reading_tag = False):
         """
-            Called when a message is recieved from gstreamer
+            Called when a message is received from gstreamer
         """
         if message.type == gst.MESSAGE_TAG and self.tag_func:
             self.tag_func(message.parse_tag())
         elif message.type == gst.MESSAGE_EOS and not self.is_paused() \
             and self.eof_func:
             self.eof_func()
+        elif message.type == gst.MESSAGE_ERROR:
+            print message, dir(message)
 
         return True
 
+    def __notify_source(self, o, s, num):
+        s = self.playbin.get_property('source')
+        s.set_property('device', num)
+        self.playbin.disconnect(self.notify_id)
+
     def play(self, uri):
         """
             Plays the specified uri
@@ -93,43 +113,26 @@ class GSTPlayer(Player):
 
         if not self.connections and not self.is_paused() and not \
             uri.find("lastfm://") > -1:
+
             self.connections.append(self.bus.connect('message', self.on_message))
             self.connections.append(self.bus.connect('sync-message::element',
                 self.on_sync_message))
 
             if '://' not in uri: 
-                if not os.path.isfile(uri.encode(xlmisc.get_default_encoding())):
-                    raise Exception('File does not exist: %s' %
-                        uri.encode(xlmisc.get_default_encoding()))
-
-                uri = 'file://%s' % uri
+                if not os.path.isfile(uri):
+                    raise Exception('File does not exist: ' + uri)
+                uri = 'file://%s' % uri # FIXME: Wrong.
             uri = uri.replace('%', '%25')
+
+            # for audio cds
+            if uri.startswith("cdda://"):
+                num = uri[uri.find('#') + 1:]
+                uri = uri[:uri.find('#')]
+                self.notify_id = self.playbin.connect('notify::source',
+                    self.__notify_source, num)
+
             self.playbin.set_property('uri', uri.encode(xlmisc.get_default_encoding()))
 
-        formats = {
-            'mp3':  'mad',
-            'flac': 'flac',
-            'ogg':  'vorbis',
-            'mpc':  'musepack',
-            'tta':  'tta',
-            'mp4':  'faad',
-            'm4a':  'faad',
-        }
-
-        ext_re = re.compile(r'\.([^\.]*)$')
-        m = ext_re.search(uri.encode(xlmisc.get_default_encoding()).lower())
-        if m:
-            ext = m.group(1)
-        else:
-            ext = ''
-
-        if ext in formats.keys():
-            plugin = formats[ext]
-            if not gst.registry_get_default().find_plugin(plugin):
-                raise Exception(_("You do not have the "
-                    "appropriate Gstreamer plugin installed to play "
-                    "this file: %(uri)s") % {'uri': uri})
-
         self.playbin.set_state(gst.STATE_PLAYING)
 
     def on_sync_message(self, bus, message):
@@ -232,7 +235,7 @@ class ExailePlayerErrorWindow(gtk.Window):
         self.scroll.add(self.view)
 
         buttons = gtk.HBox()
-        close = gtk.Button(_('Close'))
+        close = gtk.Button(stock=gtk.STOCK_CLOSE)
         buttons.pack_end(close, False, False)
         main.pack_start(buttons, False, False)
 
@@ -292,6 +295,26 @@ class ExailePlayer(GSTPlayer):
         self.eof_func = self.next
         self.current = None
 
+        self._sink_element_factories = []
+        self.add_sink_element_factory(ReplayGainElementFactory)
+        self.add_sink_element_factory(EqualizerElementFactory)
+
+    def add_sink_element_factory(self, factory):
+        """
+            adds a GstElement factory to the audio sink link
+        """
+        self._sink_element_factories.append(factory)
+        # TODO: Check if GstElements are unreferenced and cleaned up
+        # TODO: Redesign GstElement management
+        self.audio_sink = None
+
+    def del_sink_element_factory(self, factory):
+        """
+            deletes a GstElement factory from the audio sink link
+        """
+        self._sink_element_factories.remove(factory)
+        self.audio_sink = None
+
     def get_stop_track(self):
         """
             returns the stop_track (track to stop playback)
@@ -343,6 +366,9 @@ class ExailePlayer(GSTPlayer):
         elif "auto" in sink: sink = 'autoaudiosink'
         try:
             asink = gst.element_factory_make(sink)
+            if sink == 'gconfaudiosink':
+                # The "Music and Movies" profile
+                asink.set_property('profile', 1)
         except:
             xlmisc.log("Could not create sink %s.  Trying autoaudiosink." %
                 sink)
@@ -350,50 +376,15 @@ class ExailePlayer(GSTPlayer):
 
         sinkbin = gst.Bin()
         sink_elements = []
-        
-        # user does not want replaygain
-        if self.exaile.settings.get_boolean('replaygain/disabled', False):
-            xlmisc.log("Not using replaygain, disabled by the user")
 
-        # otherwise try loading replaygain
-        else:
-            replaygain = None
-
-            try:
-                replaygain = gst.element_factory_make('rgvolume')
-            except gst.PluginNotFoundError:
-                xlmisc.log("ReplayGain support requires gstreamer-plugins-bad 0.10.5")
-
-            if replaygain:
-                replaygain.set_property('album-mode',
-                    self.exaile.settings.get_boolean('replaygain/album_mode', True))
-                replaygain.set_property('pre-amp',
-                    self.exaile.settings.get_float('replaygain/preamp'))
-                replaygain.set_property('fallback-gain',
-                    self.exaile.settings.get_float('replaygain/fallback'))
-                
-                sink_elements.append(replaygain)
-                xlmisc.log("ReplayGain support initialized.")
-
-        # if the equalizer is disabled, print info
-        if self.exaile.options.noeq:
-            xlmisc.log("Not using equalizer, disabled by the user")
-
-        # otherwise try loading equalizer
-        else:
-            try: # Equalizer element is still not very common 
-                self.equalizer = gst.element_factory_make('equalizer-10bands')
-            except gst.PluginNotFoundError:
-                xlmisc.log("Equalizer support requires gstreamer-plugins-bad 0.10.5")
-
-            if self.equalizer:
-                sink_elements.append(self.equalizer)
-                sink_elements.append(gst.element_factory_make('audioconvert'))
-
-                bands = self.exaile.settings.get_list('equalizer/band-values', 
-                    [0] * 10)
-                for i, v in enumerate(bands):
-                    self.equalizer.set_property(('band' + str(i)), v)
+        # iterate through sink element factory list
+        for element_factory in self._sink_element_factories:
+            if element_factory.is_enabled(self):
+                # This should be made a try: except: statement in case creation fails
+                sink_elements += element_factory.get_elements(self)
+                xlmisc.log(element_factory.name + " support initialized.")
+            else:
+                xlmisc.log("Not using " + element_factory.name + " disabled by the user")
 
         # if still empty just use asink and end
         if not sink_elements:
@@ -422,7 +413,6 @@ class ExailePlayer(GSTPlayer):
 
     def setup_playbin(self):
         GSTPlayer.setup_playbin(self)
-        self.set_audio_sink
         self.playbin.set_property('volume',
             self.exaile.settings.get_float('volume', .7))
 
@@ -525,17 +515,26 @@ class ExailePlayer(GSTPlayer):
         if track.type == 'stream':
             track.start_time = time.time()
 
-        if track.type == 'podcast':
-            if not track.download_path:
-                common.error(self.exaile.window, _('Podcast has not yet been '
-                    'downloaded'))
-                return
+#        if track.type == 'podcast':
+#            if not track.download_path:
+#                common.error(self.exaile.window, _('Podcast has not yet been '
+#                    'downloaded'))
+#                return
 
         self.exaile.play_button.set_image(self.exaile.get_pause_image())
 
         try:
-            GSTPlayer.play(self, track.loc)
+            play_loc = track.loc
+            if track.type == 'podcast' and track.download_path:
+                play_loc = track.download_path
+
+            # custom track setup
+            if hasattr(track, 'start_play'):
+                track.start_play()
+
+            GSTPlayer.play(self, play_loc)
         except Exception, e:
+            logger.log_exception()
             self.error_window.log(str(e))
             self.error_window.show_all()
             if from_button: self.exaile.player.stop()
@@ -547,10 +546,14 @@ class ExailePlayer(GSTPlayer):
 
         self.current = track
         self.exaile.update_track_information()
-        track.submitted = False
 
         self.emit('play-track', track)
 
+        # update information tab if it's open and set to do so
+        if self.exaile.settings.get_boolean('ui/information_autoswitch',
+            False):
+            information.update_info(self.exaile.playlists_nb, track)
+
         artist_id = library.get_column_id(self.exaile.db, 'artists', 'name', track.artist)
         library.get_album_id(self.exaile.db, artist_id, track.album)
 
@@ -627,7 +630,6 @@ class ExailePlayer(GSTPlayer):
       
         track = self.exaile.tracks.get_next_track(self.current)
 
-        print 'next track was reported as ', track
         if not track:
             if not self.exaile.tracks.get_songs():
                 if not self.queued: return
@@ -699,9 +701,11 @@ class ExailePlayer(GSTPlayer):
         """
             Stops the currently playing track
         """
-        self.exaile.status.set_first(None)
-        self.exaile.cover.set_image(xl.path.get_data('images', 'nocover.png'))
-        self.exaile.cover_manager.stop_cover_thread()
+        exaile = self.exaile
+
+        exaile.status.set_first(None)
+        exaile.cover.set_image(xl.path.get_data('images', 'nocover.png'))
+        exaile.cover_manager.stop_cover_thread()
 
         if self.current: self.current.start_time = 0
         current = self.current
@@ -709,16 +713,73 @@ class ExailePlayer(GSTPlayer):
         if reset_current: self.current = None
         GSTPlayer.stop(self)
 
-        self.exaile.playing = False
-        if self.exaile.tray_icon:
-            self.exaile.tray_icon.set_tooltip(_('Exaile Music Player'))
+        exaile.playing = False
 
-        self.exaile.window.set_title(_('Exaile Music Player'))
+        title = _('Exaile Music Player')
+        exaile.window.set_title(title)
+        if exaile.tray_icon:
+            exaile.tray_icon.set_tooltip(title)
 
-        self.exaile.play_button.set_image(self.exaile.get_play_image())
-        if self.exaile.tracks: self.exaile.tracks.queue_draw()
-        self.exaile.update_track_information(None)
-        self.exaile.new_progressbar.set_text('0:00 / 0:00')
+        exaile.play_button.set_image(exaile.get_play_image())
+        if exaile.tracks: exaile.tracks.queue_draw()
+        exaile.update_track_information(None)
+        exaile.new_progressbar.set_text('0:00 / 0:00')
+
+class ReplayGainElementFactory(object):
+    name = u"ReplayGain"
+    
+    @staticmethod
+    def is_enabled(exaileplayer):
+        return exaileplayer.exaile.settings.get_boolean('replaygain/enabled', True)
+    
+    @staticmethod
+    def get_elements(exaileplayer):
+        replaygain = None
+
+        try:
+            replaygain = gst.element_factory_make('rgvolume')
+        except gst.PluginNotFoundError:
+            xlmisc.log("ReplayGain support requires gstreamer-plugins-bad 0.10.5")
+
+        if replaygain:
+            replaygain.set_property('album-mode',
+                exaileplayer.exaile.settings.get_boolean('replaygain/album_mode', True))
+            replaygain.set_property('pre-amp',
+                exaileplayer.exaile.settings.get_float('replaygain/preamp'))
+            replaygain.set_property('fallback-gain',
+                exaileplayer.exaile.settings.get_float('replaygain/fallback'))
+            
+            # Using the ugly method for returing a 1-element tuple
+            return replaygain,
+
+        return ()
+
+class EqualizerElementFactory(object):
+    name = u"Equalizer"
+    
+    @staticmethod
+    def is_enabled(exaileplayer):
+        if not exaileplayer.exaile.settings.get_boolean('equalizer/enabled',
+            False): 
+            return False
+        return not exaileplayer.exaile.options.noeq
+    
+    @staticmethod
+    def get_elements(exaileplayer):
+        try: # Equalizer element is still not very common 
+            exaileplayer.equalizer = gst.element_factory_make('equalizer-10bands')
+        except gst.PluginNotFoundError:
+            xlmisc.log("Equalizer support requires gstreamer-plugins-bad 0.10.5")
+
+        elements = []
+        if exaileplayer.equalizer:
+            elements.append(exaileplayer.equalizer)
+            elements.append(gst.element_factory_make('audioconvert'))
+
+            bands = exaileplayer.exaile.settings.get_list('equalizer/band-values', [0] * 10)
+            for i, v in enumerate(bands):
+                exaileplayer.equalizer.set_property(('band' + str(i)), v)
+        return elements
 
 # VideoWidget and VideoArea code taken from Listen media player
 # http://listen-gnome.free.fr
