@@ -20,12 +20,14 @@ This file contains every miscellanious dialog and class that is not over
 their own file
 """
 
-import httplib, os, re, sys, threading, time, traceback, urllib, urlparse
+import urllib2, os, re, sys, threading, time, traceback, urllib, urlparse
 from gettext import gettext as _
+#import gtkmozembed
 
-import pygtk
+import pygtk, locale
 pygtk.require('2.0')
-import gtk, gobject, pango
+import gtk, gobject, pango, gtk.glade
+import cairo
 from gtk.gdk import SCROLL_LEFT, SCROLL_RIGHT, SCROLL_UP, SCROLL_DOWN
 
 try:
@@ -33,17 +35,6 @@ try:
 except ImportError:
     mozembed = None
 
-USE_TRAY = None
-import warnings
-warnings.filterwarnings('ignore', 'the module egg.trayicon is deprecated',
-    DeprecationWarning)
-try:
-    import egg.trayicon
-    USE_TRAY = 'egg'
-except ImportError:
-    if hasattr(gtk, 'StatusIcon'):
-        USE_TRAY = 'gtk'
-
 try:
     import sexy
     SEXY_AVAIL = True
@@ -68,15 +59,59 @@ opener = urllib.FancyURLopener()
 opener.addheaders.pop(0)
 opener.addheader("User-Agent", "Mozilla")
 
+def check_theme_file(file):
+    """
+        Checks a theme file to make sure it's compatible with the current
+        version of Exaile.
+
+        It will open the glade file, instantiate the "ThemeInfo" dialog, and
+        check the "supported_versions" TextView for the current version.
+        Supported versions are separated by a newline.  If the current version
+        is not in the supported_versions, or the ThemeInfo dialog cannot be
+        instantiated, this function will return False
+    """
+    
+    current_version = sys.modules['__main__'].__version__
+    try:
+        xml = gtk.glade.XML(file, 'ThemeInfo', 'exaile')
+        svtv = xml.get_widget('supported_versions')
+        if not svtv: 
+            log("supported_versions widget not found, not using theme")
+            return False # the TextView could not be found
+
+        supported_versions = svtv.get_buffer().get_text(
+            svtv.get_buffer().get_start_iter(), 
+            svtv.get_buffer().get_end_iter()).split('\n')
+
+        if not current_version in supported_versions: 
+            log("Current version: %s not found in %s, not using theme" %
+                (current_version, supported_versions))
+            return False
+
+        log("%s theme is ok, continuing" % file)
+        return True
+
+    except:
+        log_exception()
+        return False
+
+
 def glade_file(exaile):
     themefile = xl.path.get_config('exaile.extheme')
     if os.path.isfile(themefile):
+        if not check_theme_file(themefile):
+            return xl.path.get_data('exaile.glade')
         return themefile
     else:
         return xl.path.get_data('exaile.glade')
 
 def get_default_encoding():
-    return 'utf-8'
+    """
+        Returns the encoding to be used when dealing with file paths.  Do not
+        use for other purposes.
+    """
+    #return 'utf-8'
+    return sys.getfilesystemencoding() or sys.getdefaultencoding()
 
 class ClearEntry(object):
     """
@@ -167,13 +202,28 @@ class MmKeys:
         try:
             import dbus
             bus = dbus.SessionBus()
-            obj = bus.get_object('org.gnome.SettingsDaemon',
-                '/org/gnome/SettingsDaemon')
-            self.__gnome = gnome = dbus.Interface(obj,
-                'org.gnome.SettingsDaemon')
-            gnome.GrabMediaPlayerKeys(self.application, 0)
-            gnome.connect_to_signal('MediaPlayerKeyPressed', on_gnome_mmkey)
+
+            try:
+                # new method (for gnome 2.22.x)
+                obj = bus.get_object('org.gnome.SettingsDaemon',
+                    '/org/gnome/SettingsDaemon/MediaKeys')
+                self.__gnome = gnome = dbus.Interface(obj,
+                    'org.gnome.SettingsDaemon.MediaKeys')
+                gnome.GrabMediaPlayerKeys(self.application, 0)
+                gnome.connect_to_signal('MediaPlayerKeyPressed', on_gnome_mmkey)
+                log("Activated gnome mmkeys for gnome 2.22.x")
+            except:
+                # old method
+                obj = bus.get_object('org.gnome.SettingsDaemon',
+                    '/org/gnome/SettingsDaemon')
+                self.__gnome = gnome = dbus.Interface(obj,
+                    'org.gnome.SettingsDaemon')
+                gnome.GrabMediaPlayerKeys(self.application, 0)
+                gnome.connect_to_signal('MediaPlayerKeyPressed', on_gnome_mmkey)
+                log("Activated gnome mmkeys for gnome 2.20.x")
+
         except:
+            log_exception()
             return None
 
         return 'gnome'
@@ -316,183 +366,6 @@ def get_icon(id, size=gtk.ICON_SIZE_BUTTON):
     return gtk.gdk.pixbuf_new_from_file(
         xl.path.get_data('images', 'default_theme', id + '.png'))
 
-class BaseTrayIcon(gobject.GObject):
-    """
-        System tray icon
-    """
-    __gsignals__ = {
-        'toggle-hide': (gobject.SIGNAL_RUN_LAST, bool, tuple())
-    }
-    def __init__(self, exaile):
-        """
-            Initializes the tray icon
-        """
-        gobject.GObject.__init__(self)
-        self.exaile = exaile
-        self.setup_menu()
-
-    def setup_menu(self):
-        """
-            Sets up the popup menu for the tray icon
-        """
-        self.menu = Menu()
-
-        self.image = gtk.Image()
-        self.image.set_from_stock('gtk-media-play',
-            gtk.ICON_SIZE_MENU)
-        self.label = gtk.Label(_("Play"))
-        self.label.set_alignment(0, 0)
-
-        self.playpause = gtk.MenuItem()
-        hbox = gtk.HBox()
-        hbox.set_spacing(5)
-        hbox.pack_start(self.image, False, True)
-        hbox.pack_start(self.label, True, True)
-        self.playpause.add(hbox)
-        self.playpause.connect('activate', lambda *e: self.exaile.player.toggle_pause())
-        self.menu.append_item(self.playpause)
-
-        self.menu.append(_("Next"), lambda *e: self.exaile.player.next(), 'gtk-media-next')
-        self.menu.append(_("Previous"), lambda *e: self.exaile.player.previous(),
-            'gtk-media-previous')
-        self.menu.append_separator()
-        self.menu.append(_("Plugins"), self.exaile.show_plugin_manager,
-            'gtk-execute')
-        self.menu.append(_("Preferences"), 
-            lambda e, a: prefs.Preferences(self.exaile).run(),
-            'gtk-preferences')
-
-        self.menu.append_separator()
-        self.menu.append(_("Quit"), self.exaile.on_quit, 'gtk-quit')
-
-    def update_menu(self):
-        track = self.exaile.player.current
-        if not track or not self.exaile.player.is_playing():
-            self.image.set_from_stock('gtk-media-play',
-                gtk.ICON_SIZE_MENU)
-            self.label.set_label(_("Play"))
-        elif self.exaile.player.is_playing():
-            self.image.set_from_stock('gtk-media-pause',
-                gtk.ICON_SIZE_MENU)
-            self.label.set_label(_("Pause"))
-
-    def toggle_exaile_visibility(self):
-        w = self.exaile.window
-        if w.is_active(): # focused
-            if self.emit('toggle-hide'): return
-            w.hide()
-        elif w.get_property('visible'): # unfocused
-            w.present()
-        else: # hidden
-            if self.emit('toggle-hide'): return
-            w.present()
-            self.exaile.setup_location()
-
-    def set_tooltip(self, tip): # to be overridden
-        """
-            Sets the tooltip for the tray icon
-        """
-        pass
-
-    def destroy(self): # to be overridden
-        """
-            Unhides the window and removes the tray icon
-
-            The unhiding is done here, while the removal needs to be
-            done in a subclass. Don't forget to call this superclass
-            method when you override it.
-        """
-        self.emit('toggle-hide') # FIXME: should this be vetoable?
-        if not self.exaile.window.get_property('visible'):
-            self.exaile.window.present()
-            self.exaile.setup_location()
-
-class EggTrayIcon(BaseTrayIcon):
-    def __init__(self, exaile):
-        BaseTrayIcon.__init__(self, exaile)
-
-        self.tips = gtk.Tooltips()
-        self.icon = egg.trayicon.TrayIcon('Exaile')
-        self.box = gtk.EventBox()
-        self.icon.add(self.box)
-
-        image = gtk.Image()
-        image.set_from_file(xl.path.get_data('images', 'trayicon.png'))
-        self.box.add(image)
-        self.box.connect('button_press_event',
-            self.button_pressed)
-        self.box.connect('scroll-event',
-            self.scroll)
-        self.box.connect('enter-notify-event', lambda *e: 
-            self.exaile.show_osd(tray=True))
-        self.icon.show_all()
-        self.set_tooltip(_("Exaile Music Player"))
-
-    def scroll(self, widget, ev):
-        """
-            Called when the user scrolls their mouse wheel over the tray icon
-        """
-        if ev.direction in [SCROLL_LEFT, SCROLL_RIGHT]:
-            ev.state = gtk.gdk.SHIFT_MASK
-        if ev.state & gtk.gdk.SHIFT_MASK:
-            if ev.direction in [SCROLL_UP, SCROLL_LEFT]: self.exaile.on_previous()
-            elif ev.direction in [SCROLL_DOWN, SCROLL_RIGHT]: self.exaile.on_next()
-        else:
-            if ev.direction in [SCROLL_RIGHT, SCROLL_UP]:
-                self.exaile.volume.page_up()
-            else:
-                self.exaile.volume.page_down()
-
-    def button_pressed(self, item, event, data=None):
-        """
-            Called when someone clicks on the icon
-        """
-        if event.button == 3:
-            self.update_menu()
-            self.menu.popup(None, None, None, event.button, event.time)
-        elif event.button == 2:
-            self.exaile.player.toggle_pause()
-        elif event.button == 1: 
-            self.toggle_exaile_visibility()
-
-    def set_tooltip(self, tip):
-        self.tips.set_tip(self.icon, tip)
-
-    def destroy(self):
-        BaseTrayIcon.destroy(self)
-        self.icon.destroy()
-
-class GtkTrayIcon(BaseTrayIcon):
-    def __init__(self, exaile):
-        BaseTrayIcon.__init__(self, exaile)
-        self.icon = icon = gtk.StatusIcon()
-        icon.set_tooltip('Exaile')
-        icon.set_from_file(xl.path.get_data('images', 'trayicon.png'))
-        icon.connect('activate', self.activated)
-        icon.connect('popup-menu', self.popup)
-        self.set_tooltip(_("Exaile Music Player"))
-
-    def activated(self, icon):
-        self.toggle_exaile_visibility()
-
-    def popup(self, icon, button, time):
-        self.update_menu()
-        self.menu.popup(None, None, gtk.status_icon_position_menu,
-            button, time, self.icon)
-
-    def set_tooltip(self, tip):
-        self.icon.set_tooltip(tip)
-
-    def destroy(self):
-        BaseTrayIcon.destroy(self)
-        self.icon.set_visible(False)
-
-if USE_TRAY == 'egg':
-    TrayIcon = EggTrayIcon
-elif USE_TRAY == 'gtk':
-    TrayIcon = GtkTrayIcon
-else:
-    TrayIcon = None
 
 class ListBox(object):
     """
@@ -659,123 +532,12 @@ class NotebookTab(gtk.EventBox):
             self.menu.popup(None, None, None, event.button, event.time)
         return False
 
-class DebugDialog(object):
-    """
-        A window to show debug messages
-    """
-    debug = None
-    
-    def __init__(self, parent):
-        """
-            Initializes the dialog
-        """
-        self.exaile = parent
-        xml = gtk.glade.XML('exaile.glade', 'DebugDialog', 'exaile')
-        self.dialog = xml.get_widget('DebugDialog')
-        self.dialog.set_transient_for(self.exaile.window)
-        self.view = xml.get_widget('debug_textview')
-        self.buf = self.view.get_buffer()
-        self.log_file = None
-        xml.get_widget('debug_ok_button').connect('clicked', 
-            lambda *e: self.dialog.hide())
-        self.dialog.connect('delete_event', self.destroy)
-        
-        DebugDialog.debug = self
-
-    def destroy(self, *e):
-        """
-            Called when the users clicks on the close button
-        """
-        self.dialog.hide()
-        return True
-
-    def log(self, message, timestamp=None):
-        """ 
-            Logs a message
-        """
-
-        if not timestamp: timestamp = time.time()
-        lt = time.localtime(timestamp)
-        text = "[%s] %s\n" % (time.strftime("%H:%M:%S", lt), message)
-        char = self.buf.get_char_count()
-        iter = self.buf.get_iter_at_offset(char + 1)
-
-        self.buf.insert(iter, text)
-        print message
-        if not self.log_file:
-            try:
-                self.log_file = open(xl.path.get_config('exaile.log'), 'a')
-            except:
-                self.log_file = None
-
-        if self.log_file:
-            self.log_file.write(text)
-            self.log_file.flush()
-
-        char = self.buf.get_char_count()
-        iter = self.buf.get_iter_at_offset(char + 1)
-        self.view.scroll_to_iter(iter, 0)
-
-    def __del__(self):
-        """
-            Closes the log file
-        """
-
-        if self.log_file:
-            self.log_file.close()
-
-LOG_QUEUE = dict()
-def log(message):
-    """
-        Queues a log event
-    """
-    gobject.idle_add(__log, message)
-
-def __log(message):
-    """
-        If the log dialog has not been initialized, it adds the message to a
-        queue to be added later, else it logs it to the debug dialog
-    """
-    dialog = DebugDialog.debug
-
-    if not dialog:
-        if media.exaile_instance:
-            DebugDialog(media.exaile_instance)
-        LOG_QUEUE[time.time()] = message
-    else:
-        if LOG_QUEUE:
-            keys = LOG_QUEUE.keys()
-            keys.sort()
-            for key in keys:
-                dialog.log(LOG_QUEUE[key], key)
-            LOG_QUEUE.clear()
-
-        dialog.log(message)
-
-def log_exception():
-    """
-        Queues a log event
-    """
-    message = log_file_and_line() + traceback.format_exc()
-    gobject.idle_add(__log_exception, message)
-
-def __log_exception(message):
-    """
-        Logs an exception
-    """
-    message = message.split("\n")
-    for line in message:
-        log(line)
-
-def log_file_and_line():
-    """
-       Logs where we are (function, file name and line number)
-       when log_exception is called... handy for debugging.
-    """
-    co = sys._getframe(2).f_code
-    return "-----------------------\n" \
-           " %s ( %s @ %s):\n" \
-           "-----------------------\n" % (co.co_name, co.co_filename, co.co_firstlineno)
+# Deprecated way for logging. Use the logger module instead.
+from xl import logger
+log = logger.log
+log_multi = logger.log_multi
+log_exception = logger.log_exception
+log_stack = logger.log_stack
 
 class URLFetcher(threading.Thread):
     """
@@ -789,7 +551,7 @@ class URLFetcher(threading.Thread):
         self.server = server 
         self.path = path 
 
-        print self.server, self.path
+        logger.log(self.server + ' ' + self.path)
         self.setDaemon(True)
         self.done_func = done_func
 
@@ -797,16 +559,16 @@ class URLFetcher(threading.Thread):
         """
             Called by thread.start()
         """
-
-        conn = httplib.HTTPConnection(self.server)
-
-        conn.request("GET", self.path)
-        response = conn.getresponse()
-
-        if response.status != 200:
+        try:
+            response = urllib2.urlopen("http://" + self.server + self.path)
+        except urllib2.URLError, e:
             self.notify("Could not load page. "
-                "Error %d: %s" % 
-                (response.status, response.reason))
+                "Error: %s" % e.reason)
+            return
+
+        except urllib2.HTTPError, e:
+            self.notify("Could not load page. "
+                "Error %d" % e.status)
             return
 
         page = response.read()
@@ -875,8 +637,6 @@ class BrowserWindow(gtk.VBox):
         exaile.status.set_first(_('Loading page...'))
         self.view.connect('net-stop', self.on_net_stop)
 
-        self.cache_dir = xl.path.get_config('cache')
-
         self.server = ''
 
         if url:
@@ -899,7 +659,7 @@ class BrowserWindow(gtk.VBox):
         """
             Called when the user presses enter in the address bar
         """
-        url = self.entry.get_text()
+        url = unicode(self.entry.get_text(), 'utf-8')
         self.load_url(url, self.action_count)
 
     def on_location_change(self, mozembed):
@@ -933,7 +693,6 @@ class BrowserWindow(gtk.VBox):
         """
             Loads a URL, either from the cache, or from the website specified
         """
-
         self.view.load_url(url)
 
         if not self.nostyles:
@@ -942,29 +701,24 @@ class BrowserWindow(gtk.VBox):
             self.entry.set_sensitive(True)
             self.entry.set_text(url)
 
-class AboutDialog(gtk.Dialog):
+def show_about_dialog(parent, version):
     """
-        An about dialog
+        Shows an about dialog
     """
-
-    def __init__(self, parent, version):
-        """
-            Initializes the dialog
-        """
-        xml = gtk.glade.XML('exaile.glade', 'AboutDialog', 'exaile')
-        self.dialog = xml.get_widget('AboutDialog')
-        logo = gtk.gdk.pixbuf_new_from_file(xl.path.get_data('images',
-            'exailelogo.png'))
-        self.dialog.set_logo(logo)
-        # HACK: GTK+ < 2.12 (2007-09-14) use set_name.
-        try:
-            self.dialog.set_program_name(_("Exaile"))
-        except AttributeError:
-            self.dialog.set_name(_("Exaile"))
-        self.dialog.set_version(str(version))
-        self.dialog.set_transient_for(parent)
-        self.dialog.run()
-        self.dialog.destroy()
+    xml = gtk.glade.XML('exaile.glade', 'AboutDialog', 'exaile')
+    dialog = xml.get_widget('AboutDialog')
+    logo = gtk.gdk.pixbuf_new_from_file(xl.path.get_data('images',
+        'exailelogo.png'))
+    dialog.set_logo(logo)
+    # HACK: GTK+ < 2.12 (2007-09-14) use set_name.
+    try:
+        dialog.set_program_name(_("Exaile"))
+    except AttributeError:
+        dialog.set_name(_("Exaile"))
+    dialog.set_version("\n" + str(version))
+    dialog.set_transient_for(parent)
+    dialog.run()
+    dialog.destroy()
 
 class ImageWidget(gtk.Image):
     """
@@ -1151,14 +905,37 @@ class OSDWindow(object):
         self.__timeout = None
         self.start_timer = start_timer
 
-        color = gtk.gdk.color_parse(settings['osd/bgcolor'])
+        self.color = gtk.gdk.color_parse(settings['osd/bgcolor'])
         self.settings = settings
         self.event = self.xml.get_widget('popup_event_box')
         self.box = self.xml.get_widget('image_box')
-        self.window.modify_bg(gtk.STATE_NORMAL, color)
-        self.title = self.xml.get_widget('popup_title_label')
+
         self.progress = self.xml.get_widget('osd_progressbar')
-        self.progress.modify_bg(gtk.STATE_NORMAL, color)
+
+        # Try to set the window opacity.  To do that we need the RGBA colormap,
+        # which for some reason may not be available even if
+        # Widget.is_composited is true.  In GTK+ >=2.12 all this can just be
+        # modified to use Window.set_opacity.
+        colormap = None
+        # Widget.is_composited is only in GTK+ >=2.10.
+        is_composited = getattr(self.window, 'is_composited', None)
+        if is_composited and is_composited():
+            screen = self.window.get_screen()
+            colormap = screen.get_rgba_colormap()
+        if colormap:
+            self.window.set_colormap(colormap)
+            self.window.set_app_paintable(True)
+            self.window.connect("expose-event", self.expose_callback)
+
+            self.progress.set_colormap(colormap)
+            self.progress.set_app_paintable(True)
+            self.progress.connect("expose-event", self.expose_callback)
+        else:
+            # Just set the background color in the old fashioned way
+            self.window.modify_bg(gtk.STATE_NORMAL, self.color)
+            self.progress.modify_bg(gtk.STATE_NORMAL, self.color)
+            
+        self.title = self.xml.get_widget('popup_title_label')
 
         self.window.set_size_request(settings['osd/w'], settings['osd/h'])
         self.cover = ImageWidget()
@@ -1170,10 +947,41 @@ class OSDWindow(object):
         self.exaile.connect('timer_update', self.timer_update)
         self.__handler = None
 
+    def expose_callback(self, widget, event):
+        cr = widget.window.cairo_create()
+        cr.set_operator(cairo.OPERATOR_SOURCE)
+
+        cr.rectangle(event.area.x, event.area.y,
+            event.area.width, event.area.height)
+        cr.clip()
+
+        opacity = self.settings['osd/opacity']
+        
+        cr.set_source_rgba(self.color.red/65535.0, self.color.green/65535.0, 
+                self.color.blue/65535.0, opacity/100.0)
+        cr.paint()
+        return False
+
     def timer_update(self, *e):
         if self.window.get_property('visible'):
             self.progress.set_fraction(self.exaile.new_progressbar.get_fraction())
 
+    def expose_callback(self, widget, event):
+        cr = widget.window.cairo_create()
+        cr.set_operator(cairo.OPERATOR_SOURCE)
+
+        cr.rectangle(event.area.x, event.area.y,
+            event.area.width, event.area.height)
+        cr.clip()
+
+        opacity = self.settings['osd/opacity']
+        
+        cr.set_source_rgba(self.color.red/65535.0, self.color.green/65535.0, 
+                self.color.blue/65535.0, opacity/100.0)
+        cr.paint()
+        return False
+
+
     def start_dragging(self, widget, event):
         """
             Called when the user starts dragging the window
@@ -1277,6 +1085,7 @@ def get_osd(exaile, settings):
 def get_osd_settings(settings):
     info = dict()
     info['osd/bgcolor'] = settings.get_str("osd/bgcolor", "#567ea2")
+    info['osd/opacity'] = settings.get_int("osd/opacity", 80)
     info['osd/w'] = settings.get_int("osd/w", 400)
     info['osd/h'] = settings.get_int("osd/h", 95)
     info['osd/y'] = settings.get_int("osd/y", 0)
@@ -1393,7 +1202,8 @@ class DragTreeView(gtk.TreeView):
             if event.type == gtk.gdk._2BUTTON_PRESS:
                 self.cont.button_press(button, event)
 
-            if selection.count_selected_rows() <= 1: return False
+            if selection.count_selected_rows() <= 1: 
+                return False
             else: 
                 if selection.path_is_selected(path[0]): 
                     if event.state & (gtk.gdk.SHIFT_MASK|gtk.gdk.CONTROL_MASK):
@@ -1464,7 +1274,7 @@ class PlaylistParser(object):
 
 class M3UParser(PlaylistParser):
     PLAYLIST_EXTS.append('.m3u')
-    REGEX = re.compile(r'#EXTINF:\d+,(.*?)[\r\n]+(.*?)[\r\n]+', re.DOTALL) 
+    REGEX = re.compile(r'#EXTINF:-?\d+,(.*?)[\r\n]+([^\r\n]+)', re.DOTALL) 
 
     def _do_parse_file(self, file):
         # Read first line to see if this is extended M3U.
@@ -1475,7 +1285,7 @@ class M3UParser(PlaylistParser):
         basedir = os.path.dirname(file.url)
         for line in [firstline] + file.readlines():
             line = line.strip()
-            if line and line[0] == "#":
+            if line and not line.startswith("#"):
                 url = self._get_url_from_path(basedir, line)
                 self.add_url(url)
 
@@ -1526,6 +1336,24 @@ class ASXParser(PlaylistParser):
         return True
 
 
+class XSPFParser(PlaylistParser):
+    PLAYLIST_EXTS.append('.xspf')
+
+    def _do_parse_file(self, file):
+        try:
+            import xml.etree.cElementTree as cETree
+        except:
+            return True
+
+        tree = cETree.ElementTree(file=file)
+        ns = "{http://xspf.org/ns/0/}"
+        tracks = tree.find("%strackList"%ns).findall("%strack"%ns)
+        for t in tracks:
+            url = t.find("%slocation"%ns).text.strip()
+            self.add_url(url)
+
+        return True
+
 
 # EXCEPTIONS
 
@@ -1542,3 +1370,5 @@ class TagUnsupportedException(Exception):
     """
     def __init__(self, tag):
         self.tag = tag
+
+
